/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package elliptic.areaproptool;

import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;


/**
 * 
 * Operations to handle and manage Euler diagrams drawn with ellipses
 *
 */


public class EllipseDiagramOps {
	
	// Data members
	public static String[] zoneLabels_Venn2 = {"a", "b", "ab"};
	public static String[] zoneLabels_Venn3 = {"a", "b", "c", "ab", "ac", "bc", "abc"};
	public static String[] zoneLabels_Venn4 = {"a", "b", "c", "d", "ab", "ac", "ad", "bc", "bd", "cd", "abc", "abd", "acd", "bcd", "abcd"};
	
	// ... loading and saving diagram to file
 	public static final String FILESTARTABSTRACTDESCRIPTION  = "ABSTRACTDESCRIPTION";
 	public static final String FILESTARTDIAGRAM = "DIAGRAM";
 	public static final String FILESTARTELLIPSES = "ELLIPSES";
 	public static final char FILESEPARATOR = '|';
 	
	
 	
 	// Methods
	
 	
	// ... related to zones
	public static int getNoOfZonesInVenn (int noOfEllipses, boolean includeEmptyZone){
		int noOfZonesInVenn = (int) Math.pow(2, noOfEllipses);
		if (!includeEmptyZone){
			noOfZonesInVenn -= 1;
		}
		return noOfZonesInVenn;
	}
	
    /* Getting zone labels -> static to be able to retrieve zone labels even if a diagram is not yet instantiated
	 * 	  Method not fully automatic -> not worth working out the zone labels because most of the diagrams will not have more than 3 ellipses 
	 * 	  => might as well define the zone labels as a hard coded array
	 */
	public static String[] getZoneLabels (int noOfEllipses, boolean includeEmptyZone){
		String[] emptyZone = {""};
		String[] zoneLabels = {}; 
		
		if (noOfEllipses == 2){
			zoneLabels = zoneLabels_Venn2;
		} else if (noOfEllipses == 3){
			zoneLabels = zoneLabels_Venn3;
		} else if (noOfEllipses == 4){
			zoneLabels = zoneLabels_Venn4;
		} else {
			return null;
		}
		
		return (!includeEmptyZone ? zoneLabels : (String[]) Utilities.concatArrays((Object[]) emptyZone, (Object[]) zoneLabels));
	}

	public static HashMap<String, Double> sortZoneAreasAsInLabelArray(String[] zoneLabels, HashMap<String, Double> unsortedZoneAreas){
	
		if (zoneLabels == null){return null;}
		if (unsortedZoneAreas == null){return null;}
		
		HashMap<String, Double> sortedZoneAreas = new HashMap<String, Double>();
		
		for (String zl : zoneLabels){
			Double zoneArea = unsortedZoneAreas.get(zl);
			sortedZoneAreas.put(zl, zoneArea);
		}
		
		return sortedZoneAreas;
	}
	public static Area[] getZonesAsAreas_3EllDiag (EllipseDiagram diagram, Point2D.Double centreOfSystem) {
		// empty zone is included (index=0) but is left empty/null
		// try to do this generic or at least, appropriate for 2 and 3 ellipses
		
		if (diagram.getEllipses().size() != 3){
			System.out.println("EllipseDiagramOps.getZonesAsAreas_3EllDiag: the diagram is not made up of 3-ellipses");
			return null;
		}
		
		ArrayList<Ellipse> diagEllipses = diagram.getEllipses();  
		
		Area areaA = null;
		Area areaB = null;
		Area areaC = null;
		
		for (Ellipse e : diagEllipses){
			if (e.label.toLowerCase().equals("a")){
				areaA = e.getShapeAreaForDisplay(centreOfSystem);
			} else if (e.label.toLowerCase().equals("b")){
				areaB = e.getShapeAreaForDisplay(centreOfSystem);
			} else if (e.label.toLowerCase().equals("c")){
				areaC = e.getShapeAreaForDisplay(centreOfSystem);
			}  
		}
	
		Area zones[] = new Area[8];

		// zone a
		zones[1] = new Area(areaA);
		zones[1].subtract(areaB);
		zones[1].subtract(areaC);

		// zone b
		zones[2] = new Area(areaB);
		zones[2].subtract(areaA);
		zones[2].subtract(areaC);

		// zone c
		zones[3] = new Area(areaC);
		zones[3].subtract(areaA);
		zones[3].subtract(areaB);
		
		// zone ab
		zones[4] = new Area(areaA);
		zones[4].intersect(areaB);
		zones[4].subtract(areaC);

		// zone ac
		zones[5] = new Area(areaA);
		zones[5].intersect(areaC);
		zones[5].subtract(areaB);

		// zone bc
		zones[6] = new Area(areaB);
		zones[6].intersect(areaC);
		zones[6].subtract(areaA);
			
		// zone abc
		zones[7] = new Area(areaA);
		zones[7].intersect(areaB);
		zones[7].intersect(areaC);
		
		return zones;
	}
	
	public static Area getSetAsArea (Ellipse e, Point2D.Double centreOfSystem) {
		// empty zone is included (index=0) but is left empty/null
		// try to do this generic or at least, appropriate for 2 and 3 ellipses
		return (e.getShapeAreaForDisplay(centreOfSystem));
	}
	public static Area[] getSetsAsAreas_3EllDiag (EllipseDiagram diagram, Point2D.Double centreOfSystem) {
		// empty zone is included (index=0) but is left empty/null
		// try to do this generic or at least, appropriate for 2 and 3 ellipses
		
		if (diagram.getEllipses().size() != 3){
			System.out.println("EllipseDiagramOps.getSetsAsAreas_3EllDiag: the diagram is not made up of 3-ellipses");
			return null;
		}
		
		ArrayList<Ellipse> diagEllipses = diagram.getEllipses();  
		
		Area areaA = null;
		Area areaB = null;
		Area areaC = null;
		
		for (Ellipse e : diagEllipses){
			if (e.label.toLowerCase().equals("a")){
				areaA = e.getShapeAreaForDisplay(centreOfSystem);
			} else if (e.label.toLowerCase().equals("b")){
				areaB = e.getShapeAreaForDisplay(centreOfSystem);
			} else if (e.label.toLowerCase().equals("c")){
				areaC = e.getShapeAreaForDisplay(centreOfSystem);
			}  
		}
		
		Area zones[] = new Area[]{areaA, areaB, areaC};
		return zones;
	}
	
	
	// ... load and save diagram (or its details and properties) from/to file 

	public static File[] getDiagFilesInDir (File dir){
		FilenameFilter filterDiagDescFiles = new FilenameFilter() {
		public boolean accept(File dir, String name) {
			return name.endsWith(".eld");
		}};
			
		return dir.listFiles(filterDiagDescFiles);
	}
	public static File[] getFinalDiagFilesInDir (File dir){
		FilenameFilter filterDiagDescFiles = new FilenameFilter() {
		public boolean accept(File dir, String name) {
			return name.endsWith("final.eld");
		}};
			
		return dir.listFiles(filterDiagDescFiles);
	}
	
	public static HashMap<String, Double> getReqAreaSpecFromLWFile(File file){
		HashMap<String, Double> areaspecs = new HashMap<String, Double>();
		try {
			BufferedReader b = new BufferedReader(new FileReader(file));
			String line = b.readLine();
			String[] colData = new String[2];
			String[] setLabels; 
			String zoneareaName = "";
			Double zonearea=0.0;
			while(line != null) {
				line = line.trim();
				if(line.equals("")) {
					line = b.readLine();
					continue;
				}
				line = line.trim();
				colData = line.split(" ");
				
				zoneareaName = "";
		        setLabels =  colData[0].split("&");
		        Arrays.sort(setLabels); //to make sure that they are sorted in alphabetical order
		        for (int j = 0; j < setLabels.length; j++) {
		        	zoneareaName += setLabels[j];
		        }
		    	zonearea = Utilities.safeParseDouble(colData[1]);
		    	areaspecs.put(zoneareaName, zonearea);
		    	
				line = b.readLine();	
			}
			b.close();
		} catch(IOException e){
			System.out.println("EllipseDiagramOps.getReqAreaSpecFromLWFile: "+e+"\n");
			System.exit(1);
		}
		
		return (areaspecs);
	}
	
	// ... load and save the file names of the diagrams from a file (assuming that the names are put one below the other in the form of a list)
	public static ArrayList<String> getDiagFileNamesFromFile (File file){
		ArrayList<String> diagFileNames = new ArrayList<String>();
		try {
			BufferedReader b = new BufferedReader(new FileReader(file));
			String line = b.readLine();
			while(line != null) {
				line = line.trim();
				if(line.equals("")) {
					line = b.readLine();
					continue;
				}
				diagFileNames.add(line);
				line = b.readLine();	
			}
			b.close();
		} catch(IOException e){
			System.out.println("EllipseDiagramOps.loadDiagFromFile: An IO exception occured when executing loadAdjacencyFile("+file+") in SimpleConcreteDiagram.java: "+e+"\n");
			System.exit(1);
		}
		
		return (diagFileNames);
	}
	
	
	
	// ... ... handling the diagram itself

	public static EllipseDiagram loadDiagFromFile (File file){
		
		ArrayList<Ellipse> ellsInDiag = new ArrayList<Ellipse>(3); //3 is just an initial capacity which will grow automatically as soon as the arraylist is full and add is invoked
		
		try {
			BufferedReader b = new BufferedReader(new FileReader(file));
			Character c = new Character(FILESEPARATOR);
			String separatorString = new String(c.toString());

			boolean readingAbstractDescription = false;
			boolean readingEllipses = false;
			String line = b.readLine();
			
			int separatorInd;
			double[] prop = new double[5];
			int i;
			
			while(line != null) {
				if(line.equals("")) {
					line = b.readLine();
					continue;
				}
				if(readingAbstractDescription && line.compareTo(FILESTARTABSTRACTDESCRIPTION)!=0 && line.compareTo(FILESTARTELLIPSES)!=0) {
				}
				if(readingEllipses && line.compareTo(FILESTARTELLIPSES)!=0){
					StringBuffer parseLine = new StringBuffer(line);
					separatorInd = 0;

					// get ellipse label
					separatorInd = parseLine.indexOf(separatorString);
					String label = parseLine.substring(0,separatorInd);
					parseLine.delete(0,separatorInd+1);

					i = 0;
					while(parseLine.length()!=0){
						separatorInd = parseLine.indexOf(separatorString);
						prop[i] = Double.parseDouble(parseLine.substring(0,separatorInd));
						parseLine.delete(0,separatorInd+1);
						i++;
					}

					ellsInDiag.add(new Ellipse(label,prop[0],prop[1],prop[2],prop[3],prop[4]));
				}
				if(line.compareTo(FILESTARTABSTRACTDESCRIPTION)==0) {
					readingAbstractDescription = true;
					readingEllipses = false;
				}
				if(line.compareTo(FILESTARTELLIPSES)==0) {
					readingEllipses = true;
				}
				line = b.readLine();
			}
			b.close();
		} catch(IOException e){
			System.out.println("EllipseDiagramOps.loadDiagFromFile: An IO exception occured when executing loadAdjacencyFile("+file+") in SimpleConcreteDiagram.java: "+e+"\n");
			System.exit(1);
		}
		
		return (new EllipseDiagram (ellsInDiag));
	}
	
	public static EllipseDiagram loadLWDiagFromFile (File file){
		
		ArrayList<Ellipse> ellsInDiag = new ArrayList<Ellipse>(3); //3 is just an initial capacity which will grow automatically as soon as the arraylist is full and add is invoked
		
		try {
			BufferedReader b = new BufferedReader(new FileReader(file));
			Character c = new Character(FILESEPARATOR);
			String separatorString = new String(c.toString());

			boolean readingAbstractDescription = false;
			boolean readingEllipses = false;
			String line = b.readLine();
			
			int separatorInd;
			double[] prop = new double[3];
			int i;
			
			while(line != null) {
				if(line.equals("")) {
					line = b.readLine();
					continue;
				}
				if(readingAbstractDescription && line.compareTo(FILESTARTABSTRACTDESCRIPTION)!=0 && line.compareTo(FILESTARTELLIPSES)!=0) {
				}
				if(readingEllipses && !line.contains("CIRCLES")){
					StringBuffer parseLine = new StringBuffer(line);
					separatorInd = 0;

					// get ellipse label
					separatorInd = parseLine.indexOf(separatorString);
					String label = parseLine.substring(0,separatorInd);
					parseLine.delete(0,separatorInd+1);

					i = 0;
					for (i=0;i<3;i++){
						separatorInd = parseLine.indexOf(separatorString);
						prop[i] = Double.parseDouble(parseLine.substring(0,separatorInd));
						parseLine.delete(0,separatorInd+1);
					}

					ellsInDiag.add(new Ellipse(label,prop[0],prop[0],prop[1],prop[2],0.0));
				}
				if(line.compareTo(FILESTARTABSTRACTDESCRIPTION)==0) {
					readingAbstractDescription = true;
					readingEllipses = false;
				}
				if(line.contains("CIRCLES")) {
					readingEllipses = true;
				}
				line = b.readLine();
			}
			b.close();
		} catch(IOException e){
			System.out.println("EllipseDiagramOps.loadLWDiagFromFile: An IO exception occured when executing loadAdjacencyFile("+file+") in SimpleConcreteDiagram.java: "+e+"\n");
			System.exit(1);
		}
		
		return (new EllipseDiagram (ellsInDiag));
	}
	
	public static String getEllipseDetailsForFile(Ellipse e){
		if (e == null){return null;}
		return (e.label + FILESEPARATOR + e.getA() + FILESEPARATOR + e.getB() + FILESEPARATOR + e.getXc() + FILESEPARATOR + e.getYc() + FILESEPARATOR + e.getRot() + FILESEPARATOR);
	}
	
	public static boolean saveDiagToFile(EllipseDiagram diagram, File file) {

		ArrayList<ConcreteContour> es_Contours = diagram.getEllipsesAsContours();
		ArrayList<Ellipse> es = diagram.getEllipses();
		
		try {
			BufferedWriter b = new BufferedWriter(new FileWriter(file));
			b.append(FILESTARTDIAGRAM);
			b.newLine();
			b.newLine();
			b.append(FILESTARTABSTRACTDESCRIPTION);
			b.newLine();
			b.append(ConcreteContour.generateAbstractDiagramFromList(es_Contours));
			b.newLine();
			b.newLine();
			b.append(FILESTARTELLIPSES);
			b.newLine();
			for (Ellipse e : es){
				b.append(getEllipseDetailsForFile(e));
				b.newLine();
			}
			b.close();
		}
		catch(IOException e) {
			System.out.println("EllipseDiagramOps.saveDiagToFile: An IO exception occured -> " + e);
			return false;
		}

		return true;
	}


	//this is the same as the one above with the only difference that it sorts out a clone of the ellipses (not to interfere with the actual ellipses) in asc order according to ellipse label
	//before saving to .eld file
	public static boolean saveDiagToFile2(EllipseDiagram diagram, File file) {

		ArrayList<ConcreteContour> es_Contours = diagram.getEllipsesAsContours();
		ArrayList<Ellipse> es = diagram.getEllipsesClone(); //clone since the ellipses are sorted in ascending order according to their ellipse label such that the original is not changed and thus original remained unsorted
		
		try {
			BufferedWriter b = new BufferedWriter(new FileWriter(file));
			b.append(FILESTARTDIAGRAM);
			b.newLine();
			b.newLine();
			b.append("ABSTRACT DESCRIPTION");
			b.newLine();
			b.append(ConcreteContour.generateAbstractDiagramFromList(es_Contours));
			b.newLine();
			b.newLine();
			b.append("ELLIPSES - zone name | horizontal axis | vertical axis | centre - x | centre - y | rotation ");
			b.newLine();
			//sort the ellipses in ascending order according to the ellipse label -> that is why it is important to get a clone of the ellipses rather than getting a reference to the ellipse list
			Collections.sort(es, new Comparator<Ellipse>() {
				public int compare(Ellipse e1, Ellipse e2) {
					return (e1.label.compareTo(e2.label));
				};});
			for (Ellipse e : es){
				b.append(getEllipseDetailsForFile(e));
				b.newLine();
			}
			b.close();
		}
		catch(IOException e) {
			System.out.println("EllipseDiagramOps.saveDiagToFile: An IO exception occured -> " + e);
			return false;
		}

		return true;
	}
	
	
	// ... ... handling area specifications for diagram
	public static HashMap<String, Double> computeAreaSpecsOfDiagInFile (File diagFile){
		EllipseDiagram diag = loadDiagFromFile(diagFile);
		if (diag.isValid()){
			// disable the next line of code, if the above validation chunk of code is enabled
			return (diag.getZoneAreas());
		} else {
			return null;
		}
	}
	
	private static class AreaSpecNamesComparator implements Comparator{
		//the index number is just after the final _ so this comparator gets that number and compares the area spec indices
		//to sort them out in descending order
        public int compare(Object obj1,Object obj2)
        {
        	String areaspecname1 = obj1.toString();
        	String areaspecname2 = obj2.toString();
        	Integer areaspec1_index = Integer.parseInt(areaspecname1.substring(areaspecname1.lastIndexOf('_')+1, areaspecname1.length()));
        	Integer areaspec2_index = Integer.parseInt(areaspecname2.substring(areaspecname2.lastIndexOf('_')+1, areaspecname2.length()));
            return areaspec1_index.compareTo(areaspec2_index);
        }
    }

	
	// ... random diagram

	public static EllipseDiagram generateARandomDiagram (int noOfElls, double a_b_min, double a_b_max, Point2D.Double axisRange_min, Point2D.Double axisRange_max){
		return generateARandomDiagram (noOfElls, a_b_min, a_b_max, axisRange_min, axisRange_max, false);
	}
	public static EllipseDiagram generateARandomDiagram (int noOfElls, double a_b_min, double a_b_max, Point2D.Double axisRange_min, Point2D.Double axisRange_max, boolean restrictToCircles){
		ArrayList<Ellipse> ellipsesInDiag = new ArrayList<Ellipse>(noOfElls);
		
		String ellipseLabel;
		for (int i = 0; i< noOfElls; i++){
			ellipseLabel = Character.toString((char)(((int)EllipseDiagram.ellipse1Label) + i));
			ellipsesInDiag.add(Ellipse.generateARandomEllipse(ellipseLabel, a_b_min, a_b_max, axisRange_min, axisRange_max, restrictToCircles));
		}
		return (new EllipseDiagram (ellipsesInDiag));
	}
	
	public static EllipseDiagram generateAValidRandomDiagram (int noOfElls, double a_b_min, double a_b_max, Point2D.Double axisRange_min, Point2D.Double axisRange_max, boolean checkValidity){
		return generateAValidRandomDiagram (noOfElls, a_b_min, a_b_max, axisRange_min, axisRange_max, false, checkValidity);
	}
	public static EllipseDiagram generateAValidRandomDiagram (int noOfElls, double a_b_min, double a_b_max, Point2D.Double axisRange_min, Point2D.Double axisRange_max, 
														      boolean restrictToCircles, boolean checkValidity){
		
		EllipseDiagram randomDiag;
		do {
			randomDiag = generateARandomDiagram(noOfElls, a_b_min, a_b_max, axisRange_min, axisRange_max, restrictToCircles);
		} while ( !( checkValidity && (randomDiag != null) && randomDiag.isValid() ) );
		// no need to check diagram.zoneAreasComputedSuccessfully because it is checked in isValid()

		return randomDiag;
	}
	
	
	// ... circle diagram represented using ellipses 
	
	public static boolean isCircleDiagram (EllipseDiagram diagram){
		// returns true only if ALL the ellipses are circles (that is semi-major axis = semi-minor axis)
		
		ArrayList<Ellipse> es = diagram.getEllipses();
		
		int noOfCircles = 0;
		
		for (Ellipse e : es){
			if (e.isCircle()){
				noOfCircles++;
			}
		}
		return (noOfCircles == es.size()); 
	}

	
	public static Point2D.Double[] getIPsBet2BaseOverlappingCircles (double d, double r1, double r2){
		double arcAngBetIPsForA = getArcAngleForOverlappingCircles(d, r1, r2);
		
		double ips_x = r1 * Math.cos(arcAngBetIPsForA/2);
		double ips_y = r1 * Math.sin(arcAngBetIPsForA/2);
		
		Point2D.Double[] ips = new Point2D.Double[2];
		ips[0] = new Point2D.Double(ips_x,  ips_y);
		ips[1] = new Point2D.Double(ips_x, -ips_y);
		
		return ips; 
	}
	
	
	public static double getArcAngleForOverlappingCircles (double d, double r1, double r2){
		// this is the equation used by Stirling Chow in his thesis and paper to find the arc angle when the leftmost circle with radius r1 
		// has centre (0,0) and the rightmost circle has centre d units away from the centre of circle 1 thus having centre (d,0)
		// => this is NOT a general solution to find the angle of any arc
		double val = (Math.pow(d,2)+Math.pow(r1,2)-Math.pow(r2,2)) / (2*r1*d);
		
		return ( 2 * Math.acos(val));
	}
	
	public static double getCircularArcAngleBetIPs (Point2D.Double[] ips, Point2D.Double centre){
			
		double ang0 = Math.abs(Math.atan(Math.abs(ips[0].y - centre.y)/Math.abs(ips[0].x - centre.x)));
		double ang1 = Math.abs(Math.atan(Math.abs(ips[1].y - centre.y)/Math.abs(ips[1].x - centre.x)));
		double arcAng = 0;
		
		if ( (Math.signum(ips[0].x-centre.x) == Math.signum(ips[1].x-centre.x)) && 
			 (Math.signum(ips[0].y-centre.y) == Math.signum(ips[1].y-centre.y))){ 
		
			arcAng = Math.abs(ang1 - ang0);
			
		} else if ((Math.signum(ips[0].x-centre.x) == Math.signum(ips[1].x-centre.x)) && 
				   (Math.signum(ips[0].y-centre.y) != Math.signum(ips[1].y-centre.y))){  
			
			arcAng = ang0 + ang1; 
			
		} else if ((Math.signum(ips[0].x-centre.x) != Math.signum(ips[1].x-centre.x)) && 
				   (Math.signum(ips[0].y-centre.y) == Math.signum(ips[1].y-centre.y))){  	
			
			arcAng = Math.PI - (ang0 + ang1); //no need Math.abs because already done that when find ang0 and ang1
			
		} else {   
			arcAng = (Math.PI - ang0) + ang1; 
		}
		
		return arcAng;
	}
	
	public static double areaOverlapCircles (double d, double r1, double r2){
		double ang1 = Math.abs(getArcAngleForOverlappingCircles(d, r1, r2)); 
		double ang2 = Math.abs(getArcAngleForOverlappingCircles(d, r2, r1)); 
		
		double areaSeg1 = getAreaOfCircleSegment(r1, ang1); 
		double areaSeg2 = getAreaOfCircleSegment(r2, ang2);
		
		return (areaSeg1 + areaSeg2);
	}
	
	public static double getAreaOfCircleSegment(double r, double ang){
		return (0.5 * Math.pow(r,2) * (ang - Math.sin(ang)));
	}
	
	public static double areaOverlapSharedZoneWith3rdCircle(Point2D.Double[] ips_ab, Point2D.Double centre_b, Point2D.Double centre_c, double r_a, double r_b, double r_c, boolean upper){
		
		// ip_ab
		Point2D.Double ip_ab;
		if (upper){
			ip_ab = (ips_ab[0].y > ips_ab[1].y) ? ips_ab[0] : ips_ab[1]; 
		} else {
			ip_ab = (ips_ab[0].y < ips_ab[1].y) ? ips_ab[0] : ips_ab[1]; 
		}
		
		// ip_ac 
		double distCentreAC = Math.sqrt(Math.pow(centre_c.x,2)+Math.pow(centre_c.y,2));
		Point2D.Double[] ips_ac_notRot = getIPsBet2BaseOverlappingCircles(distCentreAC, r_a, r_c);
		Point2D.Double ip_ac_notRot;
		Point2D.Double ip_ac_notRot2;
		// choose lower ip (smallest y) which when rotated would become the rightmost ip
		ip_ac_notRot  = (ips_ac_notRot[0].y < ips_ac_notRot[1].y) ? ips_ac_notRot[0] : ips_ac_notRot[1];
		ip_ac_notRot2 = (ips_ac_notRot[0].y < ips_ac_notRot[1].y) ? ips_ac_notRot[1] : ips_ac_notRot[0];
		
		double angBetCentresLineAndXaxis1 = GeometricOps.getPolarCoordAngForPnt(centre_c, new Point2D.Double(0.0,0.0));
		Point2D.Double ip_ac = new Point2D.Double ( (ip_ac_notRot.x*Math.cos(angBetCentresLineAndXaxis1)) - (ip_ac_notRot.y*Math.sin(angBetCentresLineAndXaxis1)),
													(ip_ac_notRot.x*Math.sin(angBetCentresLineAndXaxis1)) + (ip_ac_notRot.y*Math.cos(angBetCentresLineAndXaxis1)) ); 
		Point2D.Double ip_ac2= new Point2D.Double ( (ip_ac_notRot2.x*Math.cos(angBetCentresLineAndXaxis1)) - (ip_ac_notRot2.y*Math.sin(angBetCentresLineAndXaxis1)),
													(ip_ac_notRot2.x*Math.sin(angBetCentresLineAndXaxis1)) + (ip_ac_notRot2.y*Math.cos(angBetCentresLineAndXaxis1)) ); 
		
		
		// ip_bc
		double distCentreBC = Math.sqrt(Math.pow(centre_c.x - centre_b.x,2)+Math.pow(centre_c.y - centre_b.y,2));
		Point2D.Double[] ips_bc_notRot_noTransl = getIPsBet2BaseOverlappingCircles(distCentreBC, r_c, r_b);
		Point2D.Double ip_bc_notRot_noTransl;
		Point2D.Double ip_bc_notRot_noTransl2;
		// choose lower ip (smallest y) which when rotated would become the leftmost ip
		ip_bc_notRot_noTransl  = (ips_bc_notRot_noTransl[0].y < ips_bc_notRot_noTransl[1].y) ? ips_bc_notRot_noTransl[0] : ips_bc_notRot_noTransl[1];
		ip_bc_notRot_noTransl2 = (ips_bc_notRot_noTransl[0].y < ips_bc_notRot_noTransl[1].y) ? ips_bc_notRot_noTransl[1] : ips_bc_notRot_noTransl[0]; 
		double angBetCentresLineAndXaxis2 = GeometricOps.getPolarCoordAngForPnt(centre_b, centre_c);
		Point2D.Double ip_bc = new Point2D.Double ( (ip_bc_notRot_noTransl.x*Math.cos(angBetCentresLineAndXaxis2)) - (ip_bc_notRot_noTransl.y*Math.sin(angBetCentresLineAndXaxis2)) + centre_c.x,  
													(ip_bc_notRot_noTransl.x*Math.sin(angBetCentresLineAndXaxis2)) + (ip_bc_notRot_noTransl.y*Math.cos(angBetCentresLineAndXaxis2)) + centre_c.y);
		Point2D.Double ip_bc2 = new Point2D.Double ( (ip_bc_notRot_noTransl2.x*Math.cos(angBetCentresLineAndXaxis2)) - (ip_bc_notRot_noTransl2.y*Math.sin(angBetCentresLineAndXaxis2)) + centre_c.x, 
				                                     (ip_bc_notRot_noTransl2.x*Math.sin(angBetCentresLineAndXaxis2)) + (ip_bc_notRot_noTransl2.y*Math.cos(angBetCentresLineAndXaxis2)) + centre_c.y); 
		
		System.out.println("trying centre circle c: ("+centre_c.x+","+centre_c.y+")");
	
		System.out.println("ips with circle method: ab -> ("+ip_ab.x+","+ip_ab.y+")"); 
		System.out.println("ips with circle method: ac -> ("+ip_ac.x+","+ip_ac.y+") ("+ip_ac2.x+","+ip_ac2.y+")  transf ang: "+angBetCentresLineAndXaxis1 +" no rot: ("+ips_ac_notRot[0].x+","+ips_ac_notRot[0].y+"),("+ips_ac_notRot[1].x+","+ips_ac_notRot[1].y+")");
		System.out.println("ips with circle method: bc -> ("+ip_bc.x+","+ip_bc.y+") ("+ip_bc2.x+","+ip_bc2.y+")  transf ang: "+angBetCentresLineAndXaxis2);
		
		//prefer to use the methods for elliptic segments because this takes into consideration cases were the angle between the ips is greater than 180deg
		Ellipse e_a = new Ellipse ("", r_a, r_a, 0, 0, 0);
		double areaSegm_a = e_a.computeAreaOfEllipticSegment(new Point2D.Double[]{ip_ac, ip_ab});
		Ellipse e_b = new Ellipse ("", r_b, r_b, centre_b.x, centre_b.y, 0);
		double areaSegm_b = e_b.computeAreaOfEllipticSegment(new Point2D.Double[]{ip_ab, ip_bc});
		Ellipse e_c = new Ellipse ("", r_c, r_c, centre_c.x, centre_c.y, 0);
		double areaSegm_c = e_c.computeAreaOfEllipticSegment(new Point2D.Double[]{ip_bc, ip_ac});//{ip_bc, ip_ac});
		
		// area of inner triangle
		Triangle triangle = new Triangle(ip_ab, ip_ac, ip_bc);
		
		System.out.println("circle method: area seg a -> "+areaSegm_a); 
		System.out.println("circle method: area seg b -> "+areaSegm_b); 
		System.out.println("circle method: area seg c -> "+areaSegm_c);
		System.out.println("circle method: triangle abc -> "+triangle.computeArea()); 
		
		System.out.println("ips_ab: ("+ips_ab[0].x+","+ips_ab[0].y+") ("+ips_ab[1].x+","+ips_ab[1].y+")");
		
		// area of abc 
		return (areaSegm_a + areaSegm_b + areaSegm_c + triangle.computeArea());
	}
	

    public static double getDistBetweenCentresOfAreaPropCircles(double r_a, double r_b, double area_a, double area_b, double reqAreaAB){
				
		// bisection method
		
		double isZeroL = 1e-7; 
	
		
		double diffReqCurrAB_lower = 0;
		double diffReqCurrAB_mid = 0;
		double diffReqCurrAB_upper = 0;
		
		double currAB_lower = 0;
		double currAB_mid = 0;
		double currAB_upper = 0;	
		
		double d_lower = (r_a >= r_b)? (r_a - r_b) : (r_b - r_a); 
		double d_upper = r_a + r_b; 
		double padding = 0.5; 
		d_lower += (padding + isZeroL); 
		d_upper -= (padding + isZeroL); 
		double d_mid = 0;
		
		currAB_lower = areaOverlapCircles(d_lower, r_a, r_b);
		if (reqAreaAB > currAB_lower){  //currAB_lower is the greatest AB I can get
			return d_lower;
		}
				
		while(true){ 
			
			d_mid = (d_lower + d_upper)/2;
			
			currAB_lower = areaOverlapCircles(d_lower, r_a, r_b);
			currAB_mid = areaOverlapCircles(d_mid, r_a, r_b);	
			currAB_upper = areaOverlapCircles(d_upper, r_a, r_b);
			
			if (reqAreaAB > currAB_lower){  //currAB_lower is the greatest AB I can get
				return d_lower;
			}
			
			diffReqCurrAB_lower = reqAreaAB-currAB_lower;
			diffReqCurrAB_mid = reqAreaAB-currAB_mid;
			diffReqCurrAB_upper = reqAreaAB-currAB_upper;
			
			if (Math.abs(diffReqCurrAB_lower) <= isZeroL){
				return d_lower;
			} 
			if (Math.abs(diffReqCurrAB_mid) <= isZeroL){
				return d_mid;
			} 
			if (Math.abs(diffReqCurrAB_upper) <= isZeroL){
				return d_upper;
			} 

			if ((diffReqCurrAB_lower * diffReqCurrAB_mid) < 0){ 
				d_upper = d_mid;
			} else {      
				d_lower = d_mid;
			} 
			
			if (Math.abs(d_upper-d_lower) <= isZeroL){ 
				return d_mid;
			}
		}
	}

		
		

	
}









