/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package elliptic.areaproptool;

import java.awt.Color;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.io.File;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;


/**
 * 
 * General Utilities
 *
 */


public class Utilities {

	
	// Conversions and Parsing
	
	public static Double safeParseDouble(String doubleString) {
		double ret = 0.0;
		try {
			ret = Double.parseDouble(doubleString);
		} catch(Exception e) {
			System.out.println("Utilities.safeParseDouble: Could not parse double from String: "+doubleString);
			return null;
		}
		return ret;
	}

	public static Point2D.Double convertToPoint2D_Double (Point pnt){ 
		return (new Point2D.Double (new Double(pnt.x), new Double(pnt.y)));
	}
	
	public static double[] convertDblArrayListToArray(ArrayList<Double> ds_arraylist){
		double[] ds_array = new double[ds_arraylist.size()];
		int i = 0;
		for (Double d : ds_arraylist){
			ds_array[i] = d;
			i++;
		}
		return ds_array;
	}
	
	// ... converting a char/string(first char in the string) to ASCII int and vice-versa 
	public static int convertCharToASCIIint (char c){
		return (c);
	}	
	public static int convertCharToASCIIint (String s){
		// assuming that the string is a one-character string or else the ASCII value of the first character of the string is required
		return (convertCharToASCIIint(s.charAt(0)));
	}
	public static char convertIntASCIItoChar (int i){
		return ((char)i);
	}
	public static String convertIntASCIItoString (int i){
		return (Character.toString(convertIntASCIItoChar(i)));
	}
	
	
	// ... converting between coordinate systems
	public static Point2D.Double changeCoorSys_originMid_to_originTopLeft(Point2D.Double pnt, Point2D.Double centreOfSystem){
		return new Point2D.Double( new Double(pnt.x + centreOfSystem.x), new Double( -pnt.y + centreOfSystem.y ));
	}
	
	public static Point2D.Double changeCoorSys_originTopLeft_to_originMid(Point2D.Double pnt, Point2D.Double centreOfSystem){
		return new Point2D.Double( new Double(pnt.x - centreOfSystem.x), new Double( -pnt.y + centreOfSystem.y ));
	}
	
	public static Point2D.Double changeCoorSys_originTopLeft_to_originBottomLeft(Point2D.Double pnt, double boxHeight){
		return new Point2D.Double( new Double(pnt.x), new Double(boxHeight - pnt.y));
	}
	
	public static Point2D.Double changeCoorSys_originBottomLeft_to_originTopLeft(Point2D.Double pnt, double boxHeight){
		return new Point2D.Double( new Double(pnt.x), new Double(boxHeight - pnt.y));
	}
	
	
	// Rounding
	
	public static double roundToDps (double value, int digitsBeforeDP, int digitsAfterDP) {
		String formatAsStr = "";
		for (int i=0; i < digitsBeforeDP; i++){ formatAsStr += "#";}
		formatAsStr = ".";
		for (int i=0; i < digitsAfterDP; i++){  formatAsStr += "#";}
		
    	DecimalFormat dpForm = new DecimalFormat(formatAsStr);
	    return Double.valueOf(dpForm.format(value));
	}
	
	
	// Combinatorial Functions
	public static Double[][] generateNumPermSeq (Double[] numL){
		if ((numL == null) || (numL.length == 0)){
			return null;
		}
		
		int noOfNumPermSeq = (int)Math.pow(numL.length, numL.length);
		Double[][] numPermSeqL = new Double[noOfNumPermSeq][numL.length];
		int seqCount = 0;
		for (Double num2 : numL){
			for (Double num1 : numL){
				for (Double num0 : numL){
					numPermSeqL[seqCount][0]=num0;
					numPermSeqL[seqCount][1]=num1;
					numPermSeqL[seqCount][2]=num2;
					seqCount++;
				}
			}
		}
		return numPermSeqL;
	}
	
	public static int noOfCombinations (int n, int r){
		// nCr = n!/(((n-r)!r!))
		// different ways how to choose r out of the n 		
		return (factorial(n) / (factorial(n-r)*factorial(r)));
	}
	
	public static Integer factorial(int value) {
		// iterative version 
		
		if (value < 0){ return null;}
	
		if ((value == 0) || (value == 1)){
			return 1;
		} else {
			int result = value;
			for (int i = (value-1); i > 1; i--){
				result *= i;
			}
			return result;
		}
	}	
	
	
	
	// Time and Date
	
	public static String getCurrentDateTime(){
		DateFormat dateFormat = new SimpleDateFormat("yyyyMMdd_HHmmss");
	    Date date = new Date();
	    return(dateFormat.format(date));
	}


	
	// Colours 
	
	public static Color getColorFromChar(char c, Color colors[]) {
		int i = (int)c - (int)'a';
		if(i < 0) {i = 0-i;} // capitals give a negative i
		
		int colorInt = i % colors.length;
	    Color color = colors[colorInt];
		while(i >= colors.length) {
			color.brighter();
			i = i - colors.length;
		}
		return color;
	}
		
	public static Color mixColors(Color c1, Color c2) {
		int r1 = c1.getRed();
		int g1 = c1.getGreen();
		int b1 = c1.getBlue();

		int r2 = c2.getRed();
		int g2 = c2.getGreen();
		int b2 = c2.getBlue();

		Color ret = new Color(r1^r2, g1^g2, b1^b2);
		return ret;
	}	
	
	public static Color getRandomColor() {
		Random random = new Random();
		return new Color(random.nextInt(255),random.nextInt(255),random.nextInt(255));
	}
	
	
	
	// Random Operators

	public static Double randomNumberInRange (double range_first, double range_last){
		
		// check the given range
	    if (range_first > range_last) {
		      System.out.println("Utilities.randomNoInRange:  inappropriate range - the first number in the range is greater than the last one");
		      return null;
	    }
	    
	    Random random = new Random();
	    
	    double range = range_last - range_first + 1;
	    
	    // compute a fraction of the range, 0 <= frac < range
	    double fraction = (range * random.nextDouble());
	    double randomNo =  (fraction + range_first);  
	    
	    return (new Double (randomNo));
	}
	
	
	
	public static boolean createDir (String dirPath){
		boolean success;
		try{
			// Create all directories (if multiple directories are required according to dirPath)
			success = (new File(dirPath)).mkdirs();
			
		}catch (Exception e){ //Catch exception if any
			System.err.println("Utilities.randomNoInRange: " + e.getMessage());
			return false;
		}
		return success;
	}
	
	
	
	// Array Operators
	
	public static Object[] concatArrays (Object[] array1, Object[] array2) {
		
		Object[] array3 = new Object [array1.length + array2.length];
		
		System.arraycopy(array1, 0, array3, 0, array1.length);
		System.arraycopy(array2, 0, array3, array1.length, array2.length);

		return array3;
	}
	
	
	
	// HashMaps Operators
	
	//public static HashMap<Object, Object> sortHashMapByKeys (HashMap<Object, Object> unsortedHashMap){
	//   could not use the above generic types because of problems with HashMap type casts when invoking this method
	public static HashMap<String, Double> sortHashMapByKeys(HashMap<String, Double> unsortedHashMap){
		HashMap<String, Double> sortedHashMap = new HashMap<String, Double>();
		
		Object[] keys = unsortedHashMap.keySet().toArray();
		Arrays.sort(keys);
		
		Double value;
		for (Object key : keys){
			value = unsortedHashMap.get(key);
			sortedHashMap.put((String)key,value);
		}
		return sortedHashMap;
	}
	
	public static HashMap sortHashMapByValue(HashMap map) {
	// Obtained this from http://stackoverflow.com/questions/109383/how-to-sort-a-mapkey-value-on-the-values-in-java
	     List list = new LinkedList(map.entrySet());
	     Collections.sort(list, new Comparator() {
	          public int compare(Object o1, Object o2) {
	               return ((Comparable) ((Map.Entry) (o1)).getValue())
	              .compareTo(((Map.Entry) (o2)).getValue());
	          }
	     });

	     HashMap result = new LinkedHashMap();
	     for (Iterator it = list.iterator(); it.hasNext();) {
	    	 Map.Entry entry = (Map.Entry)it.next();
	    	 result.put(entry.getKey(), entry.getValue());
	     }
	     return result;
	}
}