/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package utils;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;


/**
 * 
 * Geometric Operations and Utilities
 *
 */


public class GeometryUtils {
  
	 
	
	public static Rectangle2D.Double[][] cloneArrayOfRectangleArrays (Rectangle2D.Double[][] rectangleArrays ){
		
		Rectangle2D.Double[][] rectangleArraysClone = new Rectangle2D.Double[rectangleArrays.length][];
		
		int rectArrayIndex =0;
		for(Rectangle2D.Double[] rectArray : rectangleArrays){
			Rectangle2D.Double[] rectArrayClone = new Rectangle2D.Double[rectArray.length];
			int rectIndex = 0;
			for(Rectangle2D.Double rect : rectArray){
				rectArrayClone[rectIndex] = new  Rectangle2D.Double(rect.x, rect.y, rect.getWidth(), rect.getHeight());
				rectIndex++;
			}
			rectangleArraysClone[rectArrayIndex] = rectArrayClone;
			rectArrayIndex++;
		}
		
		return rectangleArraysClone;
	}


	  
	 /**
	   * Computes the radius of a circle given its area 
	   * 
	   * @param area the area of the circle 
	   * @return the radius of the circle for the given area 
	   */
	  public static double computeCircleRadiusForArea(double area){
		  return (Math.abs(Math.sqrt(area / Math.PI)));
	  }
	  
	  /**
	   * Computes the area of a circle with the given radius
	   * 
	   * @param radius the radius of the circle 
	   * @return the area of the circle
	   */
	  public static double computeAreaOfCircle (double radius){
		  return (Math.PI*Math.pow(radius, 2));
	  }
	  
	  /**
	   * Computes the area of a rectangle with the given width and height
	   * 
	   * @param width the width of the rectangle 
	   * @param height the height of the rectangle 
	   * @return the area of the rectangle
	   */
	  public static double computeAreaOfRect (double width, double height){
		  return (width * height);
	  }
	  public static double computeAreaOfRect (Rectangle2D.Double rectangle){
		  return (rectangle.getWidth() * rectangle.getHeight());
	  }

	  /** 
	   * Computes the perpendicular distance of a circle's chord from the circle's centre
	   * Any circle chord is handled.
	   * 
	   * @param circleRadius the radius of the circle
	   * @param chordLength the length of the chord
	   * @return the perpendicular distance of the chord from the circle's centre 
	   */
	  public static double computePerpendicularDistanceOfCircleChordFromCentre (double circleRadius, double chordLength){		  
		  return Math.sqrt(Math.pow(circleRadius,2)-Math.pow(chordLength/2, 2));  
	  }
	  
	  
	  /** 
	   * Computes the maximum rectangle in area
	   * 
	   * @param the area
	   */
	  public static Rectangle computeMaxRectInArea(Area area) {
		 
	   	  Rectangle bbox = area.getBounds();
	 
		  // cant find a center of an area with a small or zero bounding box
		  if(bbox.width < 3) {
			  return null;
		  }
		  int divider = 3;
		  Rectangle containedRectangle = null;
		  int width; int height;
		  int xGrid; int yGrid;
		  int x; int y;
		  Rectangle rect = new Rectangle();
		  while(containedRectangle == null) {
			  width = bbox.width/divider;
			  height = bbox.height/divider;
			  // if we get to a zero size, there is no suitable rectangle
			  if(width == 0 || height == 0) {
				  return null;
			  }
			
			  for(xGrid = 1; xGrid < divider-1; xGrid++) {
				  for(yGrid = 1; yGrid < divider-1; yGrid++) {
					  x = bbox.x+xGrid*width;
					  y = bbox.y+yGrid*height;
					  rect.x = x;
					  rect.y = y;
					  rect.width = width;
					  rect.height = height;
					  if(area.contains(rect)) {
						  return rect;
					  }
				  }
			  }
			  divider++;
		}
		// should never get here
		return null;
	  }
		
	  
	  /** 
	   * Attempts to get the centre of the maximum rectangle in an area
	   * 
	   * @param the area
	   */
	  public static Point getCentreOfMaxRectInArea (Area area) {
		  
		  Rectangle rect = computeMaxRectInArea(area);
		  if(rect == null) {
			  return null;
		  }

		  // expand rectangle as far as possible
			
		  while(area.contains(rect)) {
			  rect.x--;
			  rect.width++;
		  }
		  rect.x++; // last decrement takes r out of a
		  rect.width--;
		  while(area.contains(rect)) {
			  rect.y--;
			  rect.height++;
		  }
		  rect.y++; // last decrement takes r out of a
		  rect.height--;
		  while(area.contains(rect)) {
			  rect.width++;
		  }
		  rect.width--; // last increment takes r out of a
		  while(area.contains(rect)) {
			  rect.height++;
		  }
		  rect.height--; // last increment takes r out of a
			
		  int x = rect.x+rect.width/2;
		  int y = rect.y+rect.height/2;
			
		  return new Point(x,y);
	  }
	  
	  
	  /** 
	   * Computes the maximum square in circle
	   * 
	   * @param the circle radius
	   * @param the circle centre
	   */
	  public static Rectangle2D.Double computeMaxSquareInCircle(double circleRadius, Point2D.Double circleCentre){
		  // the returned square is made of 4 squares, each made of 2 right angled triangles which touch the circle and centre at a 45 deg angle
		  // => the sides of each of these inner squares is (Math.sqrt(2)/2) * circleRadius
		  double side = 2 * ((Math.sqrt(2)/2) * circleRadius);
		  return new Rectangle2D.Double(circleCentre.x-(side/2),circleCentre.y+(side/2), side, side);
	  }
	  
	  
	  /** 
	   * Gets the dimensions of the proportional rectangle
	   * 
	   * @param the area of rectangle
	   * @param the height is to width
	   */
	  public static Dimension getDimensionOfProportionalRectangle(double area, double heightIsToWidth){
		  double width = Math.abs(Math.sqrt(area/heightIsToWidth)); 
		  double height = heightIsToWidth * width;
		  
		  Dimension dimension = new Dimension();
		  dimension.setSize(width, height);
		  
		  return dimension;
	  }
	  
	  
	  /** 
	   * Gets the area of a rectangle
	   * 
	   * @param the rectangle
	   */
	  public static Area getAreaForRectangle (Rectangle2D.Double rectangle){
		  return new Area(rectangle.getFrame());
	  }
	  
	  
	  /** 
	   * Gets the bounding box for rectangles
	   * 
	   * @param the rectangles
	   * @param whether the Cartesian coordinate system should be used
	   */
	  public static Rectangle2D.Double getBoundingBoxForRectangles(Rectangle2D.Double[] rectangles, boolean useCartesianCoordSystem){
		  // the coordinate system used depends on the input flag useCartesianCoordSystem
		  // -> if useCartesianCoordSystem = true then the origin is at the centre (x increases to the right, y increases upwards) 
		  //    else the origin is at the top left corner (x increases to the right, y increases downwards) 
		  // this will affect the returned value
		  
		  // find the x and y of the bounding box
		  Double min_x = null;
		  Double min_y = null;
		  Double max_x = null;
		  Double max_y = null;
		  
		  for (Rectangle2D.Double rect : rectangles){
			  if ((min_x == null)||(min_x > rect.x)){
				  min_x = rect.x;
			  }
			  if ((max_x == null)||(max_x < (rect.x + rect.getWidth()))){
				  max_x = rect.x + rect.getWidth();
			  }
			  
			  if (useCartesianCoordSystem){
				  if ((max_y == null)||(max_y < rect.y)){
					  max_y = rect.y;
				  }
				  if ((min_y == null)||(min_y > (rect.y - rect.getHeight()))){
					  min_y = rect.y - rect.getHeight();
				  }
			  } else {
				  if ((min_y == null)||(min_y > rect.y)){
					  min_y = rect.y;
				  }
				  if ((max_y == null)||(max_y < (rect.y + rect.getHeight()))){
					  max_y = rect.y + rect.getHeight();
				  }
			  }
		  }

		  if (useCartesianCoordSystem){
			  return new Rectangle2D.Double(min_x, max_y, Math.abs(max_x - min_x), Math.abs(max_y-min_y));
		  } else {
			  return new Rectangle2D.Double(min_x, min_y, Math.abs(max_x - min_x), Math.abs(max_y-min_y));
		  }
	  }
	  
	  
	  /** 
	   * Rotate rectangle 90 degrees clockwise
	   * 
	   * @param the rectangle
	   */
	  public static Rectangle2D.Double rotateRectangle90DegClockwise(Rectangle2D.Double rectangle){
		  return new Rectangle2D.Double(rectangle.x, rectangle.y, rectangle.getHeight(), rectangle.getWidth());
	  }
	  
	  
	  
	  // converting between coordinate systems
	  
	  /** 
	   * Convert from the Cartesian coordinate system (origin at the centre) to a system where the origin is top left
	   * 
	   * @param the point whose coordinates should be converted
	   * @param the centre of the system
	   */
	  public static Point2D.Double changeCoorSys_originMid_to_originTopLeft(Point2D.Double pnt, Point2D.Double centreOfSystem){
		  return new Point2D.Double( new Double(pnt.x + centreOfSystem.x), new Double( -pnt.y + centreOfSystem.y ));
	  }
	  
	  /** 
	   * Convert from a system where the origin is top left to the Cartesian coordinate system (origin at the centre)
	   * 
	   * @param the point whose coordinates should be converted
	   * @param the centre of the system
	   */
	  public static Point2D.Double changeCoorSys_originTopLeft_to_originMid(Point2D.Double pnt, Point2D.Double centreOfSystem){
		  return new Point2D.Double( new Double(pnt.x - centreOfSystem.x), new Double( -pnt.y + centreOfSystem.y ));
	  }
	
	  /** 
	   * Convert from a system where the origin is top left to a system where the origin is bottom left
	   * 
	   * @param the point whose coordinates should be converted
	   * @param the height of the box where the point is displayed
	   */
	  public static Point2D.Double changeCoorSys_originTopLeft_to_originBottomLeft(Point2D.Double pnt, double boxHeight){
		  return new Point2D.Double( new Double(pnt.x), new Double(boxHeight - pnt.y));
	  }
	
	  /** 
	   * Convert from a system where the origin is bottom left to a system where the origin is top left
	   * 
	   * @param the point whose coordinates should be converted
	   * @param the height of the box where the point is displayed
	   */
	  public static Point2D.Double changeCoorSys_originBottomLeft_to_originTopLeft(Point2D.Double pnt, double boxHeight){
		  return new Point2D.Double( new Double(pnt.x), new Double(boxHeight - pnt.y));
	  }
	  
}


























