/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */



package visforfallacy;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import uicomponent.vispanel.EulerEllipsePanel;
import uicomponent.vispanel.VisUtils.*;
import utils.*;
import visstyle.HeterogeneousChannelStyle;
import visstyle.HeterogeneousChannelStyleSolid;
import visstyle.VisStyle;
import elliptic.areaproptool.Ellipse;
import elliptic.areaproptool.EllipseDiagram;
import elliptic.areaproptool.EllipseDiagramOps;
import elliptic.areaproptool.MyLine;
import eulerellipse.EulerEllipseVisStyle;
import eulerellipse.EulerEllipseVisUtils;


/**
 * To instantiate, define and handle a 2-set population Euler diagram (with one population set contained in another) to prevent the neglect base rate fallacy
 * 
 */


public class EulerPop2SetsContainedForNeglectBaseRate extends VisForNeglectBaseRate{
	
	
	// Static Constants
	public static final int POP_SET = 0;
	public static final int POP_Y_SET = 1;
	public static final int TEST_Y_SET = 2;
	public static final int POP_N_EXCLUDING_AREA = 3;
	
	public static double spacingConcurrentCurves = 0.25;
	
	private static final String[] setsTitle = new String[]{"-vePop","+vePop","+veTest"};
	public static final String[] ellipsesRefLabel = new String[]{"a","b","c"}; 
	public static final String[] requiredRegionsRef = new String[]{"a", "ab", "ac", "abc"};
	
	private static final Placement[] setsLabelPlacement = {Placement.NORTH, Placement.NORTH, Placement.WEST, Placement.EAST};
	private static final Alignment[] setsLabelTextAlignment = {Alignment.CENTRE, Alignment.CENTRE, Alignment.RIGHT, Alignment.LEFT};
	
	private boolean showPopAsRect = false;
	private Placement placementOfContainedDiagram;
	
	
	// Constructors
	public EulerPop2SetsContainedForNeglectBaseRate(EulerEllipsePanel displayPanel){
		super(); 
		setDisplayPanel(displayPanel);
	}
	public EulerPop2SetsContainedForNeglectBaseRate(double baseRate, double testSensitivity, double testFalsePositive, EulerEllipsePanel displayPanel){
		setDisplayPanel(displayPanel);
		setInputProbabs(baseRate, testSensitivity, testFalsePositive);
	}
	public EulerPop2SetsContainedForNeglectBaseRate(double[] inputProbabs, EulerEllipsePanel displayPanel){
		setDisplayPanel(displayPanel);
		try {
			setInputProbabs(inputProbabs);
		} catch (Exception e) {
			System.out.println(ReflectionUtils.getClassName(1)+"."+ ReflectionUtils.getMethodName(1)+": "+e);
		}
	}
	
	
	// Overridden Getter and Setter 
	public EulerEllipsePanel getDisplayPanel (){
		return ((EulerEllipsePanel)this.displayPanel);
	}
	public void setDisplayPanel (EulerEllipsePanel displayPanel){
		this.displayPanel = displayPanel;
	}
	public String[] getSetsTitle(){
		return setsTitle;
	}
	public String[] getEllipsesRefLabel(){
		return ellipsesRefLabel;
	}
	
	
	// Other Getters and Setters 
	public boolean getShowPopAsRect(){
		return this.showPopAsRect;
	}
	public void setShowPopAsRect(boolean showPopAsRect){
		this.showPopAsRect=showPopAsRect;
	}
	public Placement getPlacementOfContainedDiagram(){
		return this.placementOfContainedDiagram;
	}
	public void setPlacementOfContainedDiagram(Placement placementOfContainedDiagram){
		this.placementOfContainedDiagram=placementOfContainedDiagram;
	}
	
	
	// Methods
	
	// ... implementing abstract methods in the parent class
	
	public VisStyle[] getDefaultVisStyle(boolean colourBlindSafe){
		return ((VisStyle[])EulerEllipseVisStyle.heterogeneousChannelStyle_3Ellipses(colourBlindSafe));
	}
	
	
	/**
	 * Generates the visualization for the fallacy problem 
	 * without checking whether all the probabilities that should be known for the problem are available or not 
	 * 
	 * {@link #generate()} carries out this check. {@link #generate()} is actually the one which is publicly accessible.
	 */
	protected void generateSafely(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) {
		
		// Get all the probabilistic values according to Bayes' rule
		double[] bayesProbabs = computeBayesRuleProbabs();
	
		
		// Obtain the details of the ellipses for the diagrams 
		
		double popsizefordiaggen = 1000;//this is different from the population size set by user for the number of glyphs => do not change this value
		
		// ... areas as probabilistic values (i.e. as values between 0 and 1)
		double[] ellipsesAreaProbab = new double[3];
		ellipsesAreaProbab[POP_SET] = 1;
		ellipsesAreaProbab[POP_Y_SET] = bayesProbabs[POP_Y];
		ellipsesAreaProbab[TEST_Y_SET] = bayesProbabs[TEST_Y_GIVEN_POP_Y] + bayesProbabs[TEST_Y_GIVEN_POP_N];
		
		double[] ellipsesAreaFreq = translateProbabsToFreqs(ellipsesAreaProbab, popsizefordiaggen); //totalPopFreq);
		
		
		// ... the semi minor and major axis -> when circles are used, the semi-minor axis of ellipse = semi-major axis of ellipse = radius of circle
		double[] ellipsesSemiAxis = new double[4];
		for (int i = 0; i < 3; i++){
			if (i==POP_SET){
				continue;
			} else {
				ellipsesSemiAxis[i] = GeometryUtils.computeCircleRadiusForArea(ellipsesAreaFreq[i]);
			}
		}
		double radius_PopN_excludingCircle = ellipsesSemiAxis[POP_Y_SET]+spacingConcurrentCurves;
		double area_PopN_excludingCircle = GeometryUtils.computeAreaOfCircle(radius_PopN_excludingCircle); 
		double area_PopN = ellipsesAreaFreq[POP_SET] - ellipsesAreaFreq[POP_Y_SET];
		ellipsesSemiAxis[POP_SET] = GeometryUtils.computeCircleRadiusForArea(area_PopN + area_PopN_excludingCircle);
		ellipsesSemiAxis[3] = radius_PopN_excludingCircle;
		//NB: to ensure exact areas due to the white space in between the -ve and +ve pop, when calculating the radius of the circle for the +ve Test 
		//    the ares of this circle = (+vePop and +veTest) + (-vePop and +veTest) + area of white space in this circle 
		//    BUT getting the area of the white space is not that trivial and considering that the regions with this white space is usually small and since we are not sure whether we will use this type of diagram 
		//        then I decided not to fix this and thus current: ares of the circle for +veTest = (+vePop and +veTest) + (-vePop and +veTest)
		//        which in the above loop would be:  ellipsesSemiAxis[i] = GeometryUtils.computeCircleRadiusForArea(ellipsesAreaFreq[i]);  where i=2=TEST_Y_SET 
		
		
		// ... rotation of ellipses -> when circles are used, radius is 0
	    double[] ellipsesRot = new double[]{0.0,0.0,0.0,0.0};
	    
	    // ... centres of ellipses 
	    Point2D.Double[] ellipsesCentre = new Point2D.Double[4];
	    ellipsesCentre[POP_SET] = new Point2D.Double(0.0,0.0);
	    
	    double distBetCentresOfInnerElls = EllipseDiagramOps.getDistBetweenCentresOfAreaPropCircles(ellipsesSemiAxis[POP_Y_SET], ellipsesSemiAxis[TEST_Y_SET], 
	    		                                                                                    ellipsesAreaFreq[POP_Y_SET], ellipsesAreaFreq[TEST_Y_SET], 
	    		                                                                                    translateProbabToFreq(bayesProbabs[TEST_Y_GIVEN_POP_Y], popsizefordiaggen));
		double widthOfBoxOf2InnerElls = ellipsesSemiAxis[POP_Y_SET] + distBetCentresOfInnerElls + ellipsesSemiAxis[TEST_Y_SET];
	    
		ellipsesCentre[POP_Y_SET] = new Point2D.Double(-(widthOfBoxOf2InnerElls/2)+ellipsesSemiAxis[POP_Y_SET],0.0);
		ellipsesCentre[TEST_Y_SET] = new Point2D.Double((widthOfBoxOf2InnerElls/2)-ellipsesSemiAxis[TEST_Y_SET],0.0);
		ellipsesCentre[POP_N_EXCLUDING_AREA] = new Point2D.Double(ellipsesCentre[POP_Y_SET].x, ellipsesCentre[POP_Y_SET].y);
		
		// ... labels for ellipses
		String[] ellipsesLabel = new String[]{ellipsesRefLabel[0], ellipsesRefLabel[1], ellipsesRefLabel[2], ellipsesRefLabel[0]}; //this.labels;	
		
		// ... ellipses for diagram
		ArrayList<Ellipse> ellipsesForDiagram = new ArrayList<Ellipse>(4);
		int eI=0;
	    ellipsesForDiagram.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
	    eI=3; // the inner ellipse for -ve pop
	    ellipsesForDiagram.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
	    eI=1;
	    ellipsesForDiagram.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
	    eI=2;
	    ellipsesForDiagram.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
		EllipseDiagram diag = new EllipseDiagram (ellipsesForDiagram);
		
		
		// ... 3 ellipses for diagram outer curves of sets => excluding the inner ellipse for -ve pop
		ArrayList<Ellipse> ellipsesForDiagramExclInnerCurveForNegPop = new ArrayList<Ellipse>(3);
		for (int i=0; i<3; i++){
			ellipsesForDiagramExclInnerCurveForNegPop.add(new Ellipse(ellipsesLabel[i], ellipsesSemiAxis[i], ellipsesSemiAxis[i], ellipsesCentre[i].x, ellipsesCentre[i].y, ellipsesRot[i]));
		}
		EllipseDiagram diag_exclInnerCurveForNegPop = new EllipseDiagram (ellipsesForDiagramExclInnerCurveForNegPop);
		
		
		
		// Show population as a rectangle
		Rectangle2D.Double rectPop = null;
		if (showPopAsRect){			
			rectPop = getRectangleForPopulation(ellipsesAreaFreq[POP_SET], ellipsesCentre[POP_SET], diag.getBoundingBoxAsRect(true));
		}
		
		// Place the contained diagram accordingly
		if (placementOfContainedDiagram != Placement.CENTRE){
			ArrayList<Ellipse> containedEllipses = new ArrayList<Ellipse>();
			containedEllipses.add(diag.getEllipses().get(1));
			containedEllipses.add(diag.getEllipses().get(2));
			containedEllipses.add(diag.getEllipses().get(3));
			
			EllipseDiagram translatedContainedDiagram; 
			
			if (showPopAsRect){
				translatedContainedDiagram = EulerEllipseVisUtils.translateDiagramInRectangle(new EllipseDiagram(containedEllipses), rectPop, placementOfContainedDiagram);
			} else {
				translatedContainedDiagram = EulerEllipseVisUtils.translateDiagramInCircle(new EllipseDiagram(containedEllipses), diag.getEllipses().get(0), placementOfContainedDiagram);
			}
			
			ArrayList<Ellipse> newDiagramEllipses = new ArrayList<Ellipse>();
			newDiagramEllipses.add(diag.getEllipses().get(0));
			newDiagramEllipses.add(translatedContainedDiagram.getEllipses().get(0));
			newDiagramEllipses.add(translatedContainedDiagram.getEllipses().get(1));
			newDiagramEllipses.add(translatedContainedDiagram.getEllipses().get(2));
			diag.setEllipses(newDiagramEllipses); 
			
			ArrayList<Ellipse> newDiagramEllipses_exclInnerCurveForNegPop = new ArrayList<Ellipse>();
			newDiagramEllipses_exclInnerCurveForNegPop.add(diag.getEllipses().get(0));
			newDiagramEllipses_exclInnerCurveForNegPop.add(translatedContainedDiagram.getEllipses().get(1));
			newDiagramEllipses_exclInnerCurveForNegPop.add(translatedContainedDiagram.getEllipses().get(2));
			diag_exclInnerCurveForNegPop.setEllipses(newDiagramEllipses_exclInnerCurveForNegPop); 
		}
		
		
		HeterogeneousChannelStyleSolid visStyleForExcRegion = ((HeterogeneousChannelStyleSolid)visStyle[0]).clone();
		visStyleForExcRegion.setFillColour(Color.white); //HeterogeneousChannelStyle.COLOUR_NULL);
		VisStyle[] visStyle_4curves = new VisStyle[]{visStyle[0], visStyleForExcRegion, visStyle[1], visStyle[2]};
		
		// Display the diagram in the panel
		getDisplayPanel().setEulerEllipseDiagram(diag);
		getDisplayPanel().setAreaPropEulerDiag(diag);
		getDisplayPanel().setContainingRecantangle(rectPop);
		getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle_4curves, new HeterogeneousChannelStyle[]{}));
		getDisplayPanel().setRegionsInDiagram(requiredRegionsRef);
		getDisplayPanel().setEllipsesRefLabel(this.getEllipsesRefLabel());
		getDisplayPanel().setShowSetsLabel(showSetLabels);
		getDisplayPanel().setSetsLabel(new String[]{this.getLabels()[1],"",this.getLabels()[0], this.getLabels()[2]}); //use 4 set labels when the inner curve for +ve pop is NOT eliminated in eulerellipsepanel 
		getDisplayPanel().setSetsLabelPlacement(setsLabelPlacement);
		getDisplayPanel().setSetsLabelTextAlignment(setsLabelTextAlignment);
		boolean showRegionSizeLabels = showRegionSizeMode != ShowRegionSizeMode.DONOT_SHOW;
		getDisplayPanel().setShowRegionsLabel(showRegionSizeLabels);
		if (showRegionSizeLabels){
			getDisplayPanel().setRegionsLabel(composeRegionSizeLabels(bayesProbabs, showRegionSizeMode, totalPopFreq));
		}
		getDisplayPanel().setShowLegend(showLegend);
		getDisplayPanel().setRectsInRegions(getLargestContainedRectsForEulerRegions(diag_exclInnerCurveForNegPop, rectPop));
			
		getDisplayPanel().refresh();
	}
	
	protected double[] composeRegionSizes(double[] bayesProbabs, ShowRegionSizeMode mode, int totalPopFreq){
	
		// the non-empty regions for this euler diagram (=> its abstract description) include:   a, ab, ac, abc
		// when a = Pop, b = Pop_y, c = Test_y
		// => return the region sizes accordingly and must be in the following order a,b,c,ab,ac,bc,abc
		
		double regionABC = bayesProbabs[TEST_Y_GIVEN_POP_Y];
		double regionAC = bayesProbabs[TEST_Y_GIVEN_POP_N];
		double regionAB = bayesProbabs[TEST_N_GIVEN_POP_Y];
		double regionA = 1 - (regionAB + regionABC + regionAC);
		
		double[] regionSizesAsProbabs = new double[]{regionA, regionAB, regionAC, regionABC};

		return mode.translateProbabsAsMode(regionSizesAsProbabs, totalPopFreq);
	}
	
	protected String[] composeRegionSizeLabels (double[] bayesProbabs, ShowRegionSizeMode mode, int totalPopFreq){
		final int DP = 2;
		final int MAX_SF_PRIOR_DP = 15;
			
		double[] regionSizes = composeRegionSizes (bayesProbabs, mode, totalPopFreq);
		double[] regionSizesRnded = new double[regionSizes.length];
		
		int i = 0;
		for (double rs : regionSizes){
			regionSizesRnded[i] = MathUtils.roundToDps(rs, MAX_SF_PRIOR_DP, DP);
			i++;
		}
		return translateValuesToStringForMode(regionSizesRnded, mode);
	}
	
	public Rectangle2D.Double getRectangleForPopulation(double area, Point2D.Double centre, Rectangle2D.Double containedEulerDiagBBox){
		// the same coordinate system as that for ellipses is used that is the origin of the system is at the centre		
		double diagBBoxHeightIsToWidth =  containedEulerDiagBBox.getHeight() / containedEulerDiagBBox.getWidth();
		Dimension rectDimension = GeometryUtils.getDimensionOfProportionalRectangle(area, diagBBoxHeightIsToWidth);
		
		return new Rectangle2D.Double(centre.x-(rectDimension.getWidth()/2), centre.y+(rectDimension.getHeight()/2), rectDimension.getWidth(), rectDimension.getHeight());
	}
	
	public Rectangle2D.Double[][] getLargestContainedRectsForEulerRegions(EllipseDiagram diagram, Rectangle2D.Double popAsRectangle){
		// this excludes the empty zone and thus it returns an array of 4 elements with rectangles for regions [a, ab, ac, abc] (4 rects for region a and 1 rect for each of the other regions)
		// this is specific for this type of diagram
		
		ArrayList<Ellipse> ellipses = diagram.getEllipses();
		Ellipse ellipseA = ellipses.get(0);
		Ellipse ellipseB = ellipses.get(1);
		Ellipse ellipseC = ellipses.get(2);
		Rectangle2D.Double[][] rects = new Rectangle2D.Double[4][];
		
		double width = 0 ;
		double height = 0;
		Point2D.Double topRightCoord = new Point2D.Double (0.0, 0.0);
		
		ArrayList<Ellipse> innerEllipses = new ArrayList<Ellipse>();
		innerEllipses.add(ellipseB);
		innerEllipses.add(ellipseC);
		Rectangle2D.Double innerDiagBBox = new Rectangle2D.Double(ellipseB.getXc()-ellipseB.getA(), ellipseB.getYc()+((ellipseB.getB()>ellipseC.getB())?ellipseB.getB():ellipseC.getB()),
				                                                  ellipseB.getA()+Math.abs(ellipseB.getXc()-ellipseC.getXc())+ellipseC.getA(), 2*((ellipseB.getB()>ellipseC.getB())?ellipseB.getB():ellipseC.getB()));
		// 1-set regions that is a and b (indexes in array: 0 and 3 respectively)
		
		// ... region a 
		rects[0] = new Rectangle2D.Double[4];
		if (showPopAsRect && (popAsRectangle!=null)){
			width = popAsRectangle.getWidth();
			height = (popAsRectangle.getHeight()/2) - (innerDiagBBox.getHeight()/2);
			rects[0][0] = new Rectangle2D.Double(ellipseA.getXc()-(width/2), ellipseA.getYc()+(popAsRectangle.getHeight()/2), width, height);
			rects[0][1] = new Rectangle2D.Double(ellipseA.getXc()-(width/2), ellipseA.getYc()-(innerDiagBBox.getHeight()/2), width, height);
			
			width = Math.abs((popAsRectangle.getWidth()/2) - (innerDiagBBox.getWidth()/2));
			height = innerDiagBBox.getHeight();
			rects[0][2] = new Rectangle2D.Double(ellipseA.getXc()-width-(innerDiagBBox.getWidth()/2), ellipseA.getYc()+(innerDiagBBox.getHeight()/2), width, height);
			rects[0][3] = new Rectangle2D.Double(ellipseA.getXc()+(innerDiagBBox.getWidth()/2), ellipseA.getYc()+(innerDiagBBox.getHeight()/2), width, height);

		} else {
			Rectangle2D.Double squareInCircleA = GeometryUtils.computeMaxSquareInCircle(ellipseA.getA(), new Point2D.Double(ellipseA.getXc(), ellipseA.getYc()));
			width = 2 * (squareInCircleA.getWidth()/2);
			height = (squareInCircleA.getHeight()/2) - (innerDiagBBox.getHeight()/2);
			rects[0][0] = new Rectangle2D.Double(ellipseA.getXc()-(width/2), ellipseA.getYc()+(squareInCircleA.getHeight()/2), width, height);
			rects[0][1] = new Rectangle2D.Double(ellipseA.getXc()-(width/2), ellipseA.getYc()-(innerDiagBBox.getHeight()/2), width, height);
			
			double[] ipCircleAwithTopOfInnerDiag_xs = ellipseA.getX(innerDiagBBox.y);
			double ipCircleAwithTopOfInnerDiag_x_left = (ipCircleAwithTopOfInnerDiag_xs[0] < ipCircleAwithTopOfInnerDiag_xs[1]) ? ipCircleAwithTopOfInnerDiag_xs[0] : ipCircleAwithTopOfInnerDiag_xs[1];
			width = Math.abs(ipCircleAwithTopOfInnerDiag_x_left - innerDiagBBox.x);
			height = innerDiagBBox.getHeight();
			rects[0][2] = new Rectangle2D.Double(ellipseA.getXc()-width-(innerDiagBBox.getWidth()/2), ellipseA.getYc()+(innerDiagBBox.getHeight()/2), width, height);
			rects[0][3] = new Rectangle2D.Double(ellipseA.getXc()+(innerDiagBBox.getWidth()/2), ellipseA.getYc()+(innerDiagBBox.getHeight()/2), width, height);
		}
		
		// 2-set regions that is ab and ac (indexes in array: 1 and 2 respectively)

		// ... region ab 
		boolean centreBInRegionABC = ( ellipseB.getXc() > (ellipseC.getXc() - ellipseC.getA()) );
		if (centreBInRegionABC){
			double distBetLeftmostEdgesOfEllipsesBandC = (ellipseC.getXc()-ellipseC.getA()) - (ellipseB.getXc()-ellipseB.getA()); 
			width = (Math.sqrt(2)/2) * distBetLeftmostEdgesOfEllipsesBandC;
			height =  2 * ( (Math.sqrt(2)/2) * distBetLeftmostEdgesOfEllipsesBandC );  // or 2 * width;
			topRightCoord.x = (ellipseC.getXc() - ellipseC.getA()) - width;
			topRightCoord.y =  ellipseB.getYc() + (height/2);			
		} else {
			width = ((Math.sqrt(2)/2) * ellipseB.getA()) + (ellipseB.getXc() - (ellipseC.getXc() - ellipseC.getA()));
			height = 2 * ((Math.sqrt(2)/2) * ellipseB.getA());
			topRightCoord.x = ellipseB.getXc() - ((Math.sqrt(2)/2) * ellipseB.getA());
			topRightCoord.y = ellipseB.getYc() + (height/2);         
		}
		rects[1] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
		
		// ... region ac 
		boolean centreCInRegionABC = ( ellipseC.getXc() < (ellipseB.getXc() + ellipseB.getA()) );
		if (centreCInRegionABC){
			double distBetRightmostEdgesOfEllipsesBandC = (ellipseC.getXc()+ellipseC.getA()) - (ellipseB.getXc()+ellipseB.getA()); 
			width = (Math.sqrt(2)/2) * distBetRightmostEdgesOfEllipsesBandC;
			height =  2 * ( (Math.sqrt(2)/2) * distBetRightmostEdgesOfEllipsesBandC );  // or 2 * width;
			topRightCoord.x = ellipseB.getXc() +  ellipseB.getA();
			topRightCoord.y = ellipseB.getYc() + (height/2);			
		} else {
			width = ((Math.sqrt(2)/2) * ellipseC.getA()) + (ellipseC.getXc() - (ellipseB.getXc() + ellipseB.getA()));
			height = 2 * ((Math.sqrt(2)/2) * ellipseC.getA());
			topRightCoord.x = ellipseB.getXc() +  ellipseB.getA();
			topRightCoord.y = ellipseB.getYc() + (height/2);   
		}
		rects[2] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
	
		// ... region abc
		double widthOfIntersectRegionABC = Math.abs((ellipseB.getXc()+ellipseB.getA()) - (ellipseC.getXc()-ellipseC.getA()));
		double centreRegionABC_x = (ellipseC.getXc()-ellipseC.getA()) + (widthOfIntersectRegionABC/2);
		double centreRegionABC_y =ellipseB.getYc();
		
		MyLine line45degFromIntersectCentreForEllB = new MyLine(1, -centreRegionABC_x, false);
		ArrayList<Point2D.Double> ipsEllipseBWithLine = ellipseB.getIntPnts_WithLine(line45degFromIntersectCentreForEllB);
		Point2D.Double ipEllipseBWithLine = (ipsEllipseBWithLine.get(0).getX() > ipsEllipseBWithLine.get(1).getX()) ? ipsEllipseBWithLine.get(0) : ipsEllipseBWithLine.get(1);
		double distIPEllipseBWithLineToCentre_x = Math.abs(ipEllipseBWithLine.x - centreRegionABC_x);
		double distIPEllipseBWithLineToCentre_y = Math.abs(ipEllipseBWithLine.y - centreRegionABC_y);
		
		MyLine line45degFromIntersectCentreForEllC = new MyLine(-1, centreRegionABC_x, false);
		ArrayList<Point2D.Double> ipsEllipseCWithLine = ellipseC.getIntPnts_WithLine(line45degFromIntersectCentreForEllC);
		Point2D.Double ipEllipseCWithLine = (ipsEllipseCWithLine.get(0).getX() < ipsEllipseCWithLine.get(1).getX()) ? ipsEllipseCWithLine.get(0) : ipsEllipseCWithLine.get(1);
		double distIPEllipseCWithLineToCentre_x = Math.abs(ipEllipseCWithLine.x - centreRegionABC_x);
		double distIPEllipseCWithLineToCentre_y = Math.abs(ipEllipseCWithLine.y - centreRegionABC_y);
		
		if (distIPEllipseCWithLineToCentre_y < distIPEllipseBWithLineToCentre_y){
			double[] ellipseB_xs = ellipseB.getX(centreRegionABC_y+distIPEllipseCWithLineToCentre_y);
			width = distIPEllipseCWithLineToCentre_x + Math.abs(((ellipseB_xs[0] > ellipseB_xs[1]) ? ellipseB_xs[0] : ellipseB_xs[1]) - centreRegionABC_x);
			height = 2*distIPEllipseCWithLineToCentre_y;
			topRightCoord.x = centreRegionABC_x - distIPEllipseCWithLineToCentre_x;
		} else {
			double[] ellipseC_xs = ellipseC.getX(centreRegionABC_y+distIPEllipseBWithLineToCentre_y);
			double distCentreABCtoEllipseC = Math.abs(((ellipseC_xs[0] < ellipseC_xs[1]) ? ellipseC_xs[0] : ellipseC_xs[1]) - centreRegionABC_x);
			width = distIPEllipseBWithLineToCentre_x + distCentreABCtoEllipseC;
			height = 2*distIPEllipseBWithLineToCentre_y;
			topRightCoord.x = centreRegionABC_x - distCentreABCtoEllipseC;
		}
		topRightCoord.y = centreRegionABC_y + (height/2); 
		rects[3] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
		return rects;
	}
	
}
