/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visforfallacy;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Arrays;

import uicomponent.vispanel.EulerEllipsePanel;
import uicomponent.vispanel.VisUtils.*;
import utils.GeometryUtils;
import utils.MathUtils;
import utils.TypingUtils;
import visstyle.HeterogeneousChannelStyle;
import visstyle.HeterogeneousChannelStyleOutline;
import visstyle.VisStyle;
import elliptic.areaproptool.MyLine;
import elliptic.areaproptool.Ellipse;
import elliptic.areaproptool.EllipseDiagram;
import elliptic.areaproptool.EllipseDiagramOps;
import eulerellipse.EulerEllipseVisStyle;
import eulerellipse.EulerEllipseVisUtils;


/**
 * To instantiate, define and handle a 2-set population Euler diagram to prevent the neglect base rate fallacy
 * 
 */


public class EulerPop2SetsForNeglectBaseRate extends VisForNeglectBaseRate{
	
	// Static Constants
	public static final int POP_Y_SET = 0;
	public static final int POP_N_SET = 1;
	public static final int TEST_Y_SET = 2;
	
	private static final String[] setsTitle = new String[]{"+vePop","-vePop","+veTest"};
	public static final String[] ellipsesRefLabel = new String[]{"a","b","c"}; 
	public static final String[] requiredRegionsRef = new String[]{"a", "b", "ac", "bc"};
	public static final double diagramCentreY = 0.0;
	
	protected static final Placement[] setsLabelPlacement = {Placement.WEST, Placement.EAST, Placement.NORTH};
	protected static final Alignment[] setsLabelTextAlignment = {Alignment.RIGHT, Alignment.LEFT, Alignment.CENTRE};
	
	
	// Constructors
	public EulerPop2SetsForNeglectBaseRate(EulerEllipsePanel displayPanel){
		super(); 
		setDisplayPanel(displayPanel);
	}
	public EulerPop2SetsForNeglectBaseRate(double baseRate, double testSensitivity, double testFalsePositive, EulerEllipsePanel displayPanel){
		setDisplayPanel(displayPanel);
		setInputProbabs(baseRate, testSensitivity, testFalsePositive);
	}
	public EulerPop2SetsForNeglectBaseRate(double[] inputProbabs, EulerEllipsePanel displayPanel){
		setDisplayPanel(displayPanel);
		try {
			setInputProbabs(inputProbabs);
		} catch (Exception e) {
		}
	}
	
	
	// Overridden Getter and Setter 
	public EulerEllipsePanel getDisplayPanel (){
		return ((EulerEllipsePanel)this.displayPanel);
	}
	public void setDisplayPanel (EulerEllipsePanel displayPanel){
		this.displayPanel = displayPanel;
	}
	public String[] getSetsTitle(){
		return setsTitle;
	}
	public String[] getEllipsesRefLabel(){
		return ellipsesRefLabel;
	}
	
	
	
	// Methods
	
	// ... implementing abstract methods in the parent class
	
	public VisStyle[] getDefaultVisStyle(boolean colourBlindSafe){
		return ((VisStyle[])EulerEllipseVisStyle.heterogeneousChannelStyle_3Ellipses(colourBlindSafe));
	}
	
	
	/**
	 * Generates the visualization for the fallacy problem 
	 * without checking whether all the probabilities that should be known for the problem are available or not 
	 * 
	 * {@link #generate()} carries out this check. {@link #generate()} is actually the one which is publicly accessible.
	 */
	protected void generateSafely(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) {
	    
		initBasicValues();
		
	    // Display panel settings 
		getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
		getDisplayPanel().setRegionsInDiagram(requiredRegionsRef);
		getDisplayPanel().setEllipsesRefLabel(this.getEllipsesRefLabel());
		getDisplayPanel().setShowSetsLabel(showSetLabels);
		getDisplayPanel().setSetsLabel(this.getLabels());
		getDisplayPanel().setSetsLabelPlacement(setsLabelPlacement);
		getDisplayPanel().setSetsLabelTextAlignment(setsLabelTextAlignment);
		boolean showRegionSizeLabels = showRegionSizeMode != ShowRegionSizeMode.DONOT_SHOW;
		getDisplayPanel().setShowRegionsLabel(showRegionSizeLabels);
		if (showRegionSizeLabels){
			getDisplayPanel().setRegionsLabel(composeRegionSizeLabels(bayesProbabs, showRegionSizeMode, totalPopFreq));
		}
		getDisplayPanel().setShowLegend(showLegend);
		
		EllipseDiagram diag = computeAreaProportionalEulerDiagram(bayesProbabs);
		EulerEllipseVisUtils.scaleToFit(diag, getDisplayPanel().getBoundingBox(), getDisplayPanel().getInnerPadding(), getDisplayPanel().getCentreOfPanel()); 
		
		getDisplayPanel().setEulerEllipseDiagram(diag);
		getDisplayPanel().setRectsInRegions(getLargestContainedRectsForEulerRegions(diag));
		getDisplayPanel().setAreaPropEulerDiag(diag);
		getDisplayPanel().refresh();
	}	
	
	
	protected EllipseDiagram computeAreaProportionalEulerDiagram(){
		// Get all the probabilistic values according to Bayes' rule
		double[] bayesProbabs = computeBayesRuleProbabs();
		return computeAreaProportionalEulerDiagram(bayesProbabs);
	}
	
	protected EllipseDiagram computeAreaProportionalEulerDiagram(double[] bayesProbabs){

		// Obtain the details of the ellipses for the diagrams 
		
		double popsizefordiaggen = 10000;//this is different from the population size set by user for the number of glyphs => do not change this value; this must be the same as the value in generateSafely of EulerPop1SetForNeglectBaseRate, EulerPop2SetsForNeglectBaseRate, EulerPop2SetsContainedForNeglectBaseRate as it is used to generate the diagram with no glyphs 
		
		double[] ellipsesAreaFreq = computeEllipsesAreaAsFreqs(bayesProbabs, (int)popsizefordiaggen);
		
		// ... the semi minor and major axis -> when circles are used, the semi-minor axis of ellipse = semi-major axis of ellipse = radius of circle
		double[] ellipsesSemiAxis = new double[3];
		ellipsesSemiAxis[POP_Y_SET] = GeometryUtils.computeCircleRadiusForArea(ellipsesAreaFreq[POP_Y_SET]);
		ellipsesSemiAxis[POP_N_SET] = GeometryUtils.computeCircleRadiusForArea(ellipsesAreaFreq[POP_N_SET]);
		ellipsesSemiAxis[TEST_Y_SET] = GeometryUtils.computeCircleRadiusForArea(ellipsesAreaFreq[TEST_Y_SET]);
		
		// ... rotation of ellipses -> when circles are used, radius is 0
	    double[] ellipsesRot = new double[]{0.0,0.0,0.0};
	    
	    // ... centres of ellipses 
	    Point2D.Double[] ellipsesCentre = new Point2D.Double[2];
	    
	    double boundingBoxForPopSets_width = (2*ellipsesSemiAxis[POP_Y_SET]) + (2*ellipsesSemiAxis[POP_N_SET]);
	    
	    ellipsesCentre[POP_Y_SET] = new Point2D.Double(-(boundingBoxForPopSets_width/2) + ellipsesSemiAxis[POP_Y_SET], diagramCentreY);
	    ellipsesCentre[POP_N_SET] = new Point2D.Double( (boundingBoxForPopSets_width/2) - ellipsesSemiAxis[POP_N_SET], diagramCentreY);
	    
		// ... labels for ellipses
		String[] ellipsesLabel = ellipsesRefLabel; //this.labels;	
		
		EllipseDiagram diag = computeAreaProportionalEulerDiagram_EllipseCoption1(ellipsesLabel, ellipsesCentre, ellipsesSemiAxis, ellipsesRot, ellipsesAreaFreq, bayesProbabs, popsizefordiaggen, false, false);  
		
		return diag;
	}
	
    protected double[] computeEllipsesAreaAsFreqs (double[] bayesProbabs, int popsize){
		
		double[] ellipsesAreaProbab = new double[3];
		ellipsesAreaProbab[POP_Y_SET] = bayesProbabs[POP_Y];
		ellipsesAreaProbab[POP_N_SET] = bayesProbabs[POP_N];
		ellipsesAreaProbab[TEST_Y_SET] = bayesProbabs[TEST_Y_GIVEN_POP_Y] + bayesProbabs[TEST_Y_GIVEN_POP_N];
		
		double[] ellipsesAreaFreq = translateProbabsToFreqs(ellipsesAreaProbab, popsize);
		
		return ellipsesAreaFreq;
	}
	
	
	private EllipseDiagram computeAreaProportionalEulerDiagram_EllipseCoption1(String[] ellipsesLabel, Point2D.Double[] ellipsesCentre, double[] ellipsesSemiAxis, double[] ellipsesRot, double[] ellipsesAreaFreq, double[] bayesProbabs, double popsize, boolean mapTestYSets, boolean showInitCirclesToGetEllipseTestY){  
	    // OPTION1: 
	    //    Considering the Euler-3 as 2 Venn-2 diagrams with set Test_y represented as a circle with area Test_y and duplicated 
	    //    bisection is used to get the appropriate areas of PopY+TestY and PopN+TestY
		
		EllipseDiagram retDiag = null;
		
		double reqPopYTestYfreq = translateProbabToFreq(bayesProbabs[TEST_Y_GIVEN_POP_Y], popsize);
		double reqPopNTestYfreq = translateProbabToFreq(bayesProbabs[TEST_Y_GIVEN_POP_N], popsize);
		
	    double distBetCentresOfLeftElls = EllipseDiagramOps.getDistBetweenCentresOfAreaPropCircles(ellipsesSemiAxis[POP_Y_SET], ellipsesSemiAxis[TEST_Y_SET], 
                																					ellipsesAreaFreq[POP_Y_SET], ellipsesAreaFreq[TEST_Y_SET], 
                																					reqPopYTestYfreq);
	    double centreOfTestY_popy_x = ellipsesCentre[POP_Y_SET].x + distBetCentresOfLeftElls;
	    
	    
	    double distBetCentresOfRightElls = EllipseDiagramOps.getDistBetweenCentresOfAreaPropCircles(ellipsesSemiAxis[POP_N_SET], ellipsesSemiAxis[TEST_Y_SET], 
																								   ellipsesAreaFreq[POP_N_SET], ellipsesAreaFreq[TEST_Y_SET], 
																								   reqPopNTestYfreq); ;
	    double centreOfTestY_popn_x = ellipsesCentre[POP_N_SET].x - distBetCentresOfRightElls;
	    

	    // OPTION1a: drawing set PopY next to PopN st they touch at one point and => 2 sets for TestY are visible 
	   
		if (mapTestYSets){
			ArrayList<Ellipse> ellipsesForDiagram = new ArrayList<Ellipse>(3);
			ellipsesForDiagram.add(new Ellipse(ellipsesLabel[POP_Y_SET], ellipsesSemiAxis[POP_Y_SET], ellipsesSemiAxis[POP_Y_SET], ellipsesCentre[POP_Y_SET].x, ellipsesCentre[POP_Y_SET].y, ellipsesRot[POP_Y_SET]));
			ellipsesForDiagram.add(new Ellipse(ellipsesLabel[POP_N_SET], ellipsesSemiAxis[POP_N_SET], ellipsesSemiAxis[POP_N_SET], centreOfTestY_popy_x+distBetCentresOfRightElls, ellipsesCentre[POP_N_SET].y, ellipsesRot[POP_N_SET]));
			ellipsesForDiagram.add(new Ellipse(ellipsesLabel[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET], centreOfTestY_popy_x, 0.0, 0.0));
			EllipseDiagram diag = new EllipseDiagram (ellipsesForDiagram);
			retDiag = diag;
			
			// Display the diagram in the panel
			getDisplayPanel().setEulerEllipseDiagram(diag);
			getDisplayPanel().setSetsLabel(new String[]{"+vePop", "-vePop", "+veTest"});

		}else {
			
			ArrayList<Ellipse> popEllipses = new ArrayList<Ellipse>(2);
			for (int eI=0; eI<2; eI++){
				popEllipses.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
			}
			
			Ellipse circleTestYforPopY = new Ellipse(ellipsesLabel[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET], centreOfTestY_popy_x, 0.0, 0.0);
			Ellipse circleTestYforPopN = new Ellipse(ellipsesLabel[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET], centreOfTestY_popn_x, 0.0, 0.0);
			
			if (showInitCirclesToGetEllipseTestY){
				
				((EulerEllipsePanel)displayPanel).setSetBackground(false);
				
			    // DRAWING DIAGRAM WITH TESTy FOR POPy
		
				// ... ellipses for diagram
				ArrayList<Ellipse> ellipsesForDiagram = new ArrayList<Ellipse>(3);
				ellipsesForDiagram.add(popEllipses.get(POP_Y_SET));
				ellipsesForDiagram.add(popEllipses.get(POP_N_SET));
				ellipsesForDiagram.add(circleTestYforPopY);
				EllipseDiagram diag = new EllipseDiagram (ellipsesForDiagram);
				
				// Display the diagram in the panel
				getDisplayPanel().setEulerEllipseDiagram(diag);
				getDisplayPanel().setSetsLabel(new String[]{"+vePop", "-vePop", "+veTest (for +vePop)"});
				getDisplayPanel().refresh();
				
				
			    // DRAWING DIAGRAM WITH TESTy FOR POPn
				ArrayList<Ellipse> ellipsesForDiagram2 = new ArrayList<Ellipse>(3);
				ellipsesForDiagram2.add(popEllipses.get(POP_Y_SET));
				ellipsesForDiagram2.add(popEllipses.get(POP_N_SET));
				ellipsesForDiagram2.add(circleTestYforPopN);
				EllipseDiagram diag2 = new EllipseDiagram (ellipsesForDiagram2);
				
				// Display the diagram in the panel
				getDisplayPanel().setEulerEllipseDiagram(diag2);
				((HeterogeneousChannelStyle)visStyle[2]).setOutlineColour(Color.BLACK);
				getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
				getDisplayPanel().setSetsLabel(new String[]{"+vePop", "-vePop", "+veTest (for -vePop)"});
				setsLabelPlacement[2]=Placement.SOUTH;
				getDisplayPanel().setSetsLabelPlacement(setsLabelPlacement);
				getDisplayPanel().refresh();
				
				
			    // DRAWING FITTING INTUITIVE ELLIPSE ON TOP
				double distBetweenCentreOfEllipseCAndCA = Math.abs((centreOfTestY_popy_x - centreOfTestY_popn_x)/2);
				ArrayList<Ellipse> ellipsesForDiagram3 = new ArrayList<Ellipse>(3);
				ellipsesForDiagram3.add(popEllipses.get(POP_Y_SET));
				ellipsesForDiagram3.add(popEllipses.get(POP_N_SET));
				ellipsesForDiagram3.add(new Ellipse(ellipsesLabel[TEST_Y_SET], ellipsesSemiAxis[TEST_Y_SET]+distBetweenCentreOfEllipseCAndCA, ellipsesSemiAxis[TEST_Y_SET], centreOfTestY_popy_x+distBetweenCentreOfEllipseCAndCA, 0.0, 0.0));		
				EllipseDiagram diag3 = new EllipseDiagram (ellipsesForDiagram3);
				
				// Display the diagram in the panel
				getDisplayPanel().setEulerEllipseDiagram(diag3);
				visStyle[2]= new HeterogeneousChannelStyleOutline("c");
				((HeterogeneousChannelStyleOutline)visStyle[2]).setOutlineColour(Color.pink);
				getDisplayPanel().setSetsLabel(new String[]{"", "", ""});
				getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
				getDisplayPanel().refresh();
			}
			
		    // DRAWING BEST FITTING ELLIPSE ON TOP
			ArrayList<Ellipse> ellipsesForDiagramDefault = new ArrayList<Ellipse>(3);
			ellipsesForDiagramDefault.add(popEllipses.get(POP_Y_SET));
			ellipsesForDiagramDefault.add(popEllipses.get(POP_N_SET));
			ellipsesForDiagramDefault.add(getBestEllipseEnclosing2Regions(new Ellipse[]{ellipsesForDiagramDefault.get(POP_Y_SET),ellipsesForDiagramDefault.get(POP_N_SET)}, 
																	new Ellipse[]{circleTestYforPopY, circleTestYforPopN}, ellipsesLabel[TEST_Y_SET], reqPopYTestYfreq, reqPopNTestYfreq));
			EllipseDiagram diagDefault = new EllipseDiagram (ellipsesForDiagramDefault);
			
			retDiag = diagDefault;
			
			// Display the diagram in the panel
			getDisplayPanel().setEulerEllipseDiagram(diagDefault);
			if (showInitCirclesToGetEllipseTestY){
				visStyle[2]= new HeterogeneousChannelStyleOutline("c");
				((HeterogeneousChannelStyleOutline)visStyle[2]).setOutlineColour(Color.green);
				getDisplayPanel().setSetsLabel(new String[]{"", "", ""});
				getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
			}
		}
		((EulerEllipsePanel)displayPanel).setSetBackground(true);
		
		return retDiag;
	}
	
	private void computeAreaProportionalEulerDiagram_EllipseCoption2(String[] ellipsesLabel, Point2D.Double[] ellipsesCentre, double[] ellipsesSemiAxis, double[] ellipsesRot, double[] ellipsesAreaFreq, double[] bayesProbabs, double popsize){  
		// OPTION 2:
		//    represent the areas of PopY+TestY and PopN+TestY as 2 circles => as disconnect zones
		
		
		// DRAWING DIAGRAM WITH TESTy FOR POPy
		
		double radiusCircleTestYPopY = GeometryUtils.computeCircleRadiusForArea(translateProbabToFreq(bayesProbabs[TEST_Y_GIVEN_POP_Y],popsize));
		// ... ellipses for diagram
		ArrayList<Ellipse> ellipsesForDiagram = new ArrayList<Ellipse>(3);
		for (int eI=0; eI<2; eI++){
			ellipsesForDiagram.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
		}
		ellipsesForDiagram.add(new Ellipse(ellipsesLabel[TEST_Y_SET], radiusCircleTestYPopY, radiusCircleTestYPopY, (ellipsesCentre[POP_Y_SET].x+ellipsesSemiAxis[POP_Y_SET])-radiusCircleTestYPopY, 0.0, 0.0));
		EllipseDiagram diag = new EllipseDiagram (ellipsesForDiagram);
		
		// Display the diagram in the panel
		getDisplayPanel().setEulerEllipseDiagram(diag);
		getDisplayPanel().setSetsLabel(new String[]{"+vePop", "-vePop", "+veTest (for +vePop)"});
		getDisplayPanel().refresh();
		
		
	    // DRAWING DIAGRAM WITH TESTy FOR POPn
		double radiusCircleTestYPopN = GeometryUtils.computeCircleRadiusForArea(translateProbabToFreq(bayesProbabs[TEST_Y_GIVEN_POP_N],popsize));
		ArrayList<Ellipse> ellipsesForDiagram2 = new ArrayList<Ellipse>(3);
		for (int eI=0; eI<2; eI++){
			ellipsesForDiagram2.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
		}
		ellipsesForDiagram2.add(new Ellipse(ellipsesLabel[TEST_Y_SET], radiusCircleTestYPopN,radiusCircleTestYPopN, (ellipsesCentre[POP_N_SET].x-ellipsesSemiAxis[POP_N_SET])+radiusCircleTestYPopN, 0.0, 0.0));
		EllipseDiagram diag2 = new EllipseDiagram (ellipsesForDiagram2);
		getDisplayPanel().setEulerEllipseDiagram(diag2);
		((HeterogeneousChannelStyle)visStyle[2]).setOutlineColour(Color.BLACK);
		getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
		getDisplayPanel().setSetsLabel(new String[]{"+vePop", "-vePop", "+veTest"});
		setsLabelPlacement[2]=Placement.SOUTH;
		getDisplayPanel().setSetsLabelPlacement(setsLabelPlacement);
		getDisplayPanel().refresh();
		
		
	    // DRAWING FITTING ELLIPSE
		ArrayList<Ellipse> ellipsesForDiagram3 = new ArrayList<Ellipse>(3);
		for (int eI=0; eI<2; eI++){
			ellipsesForDiagram3.add(new Ellipse(ellipsesLabel[eI], ellipsesSemiAxis[eI], ellipsesSemiAxis[eI], ellipsesCentre[eI].x, ellipsesCentre[eI].y, ellipsesRot[eI]));
		}
		double ellipseHorizSemiAxis = ((2*radiusCircleTestYPopY) + (2*radiusCircleTestYPopN))/2;
		ellipsesForDiagram3.add(new Ellipse(ellipsesLabel[TEST_Y_SET], ellipseHorizSemiAxis, Math.max(radiusCircleTestYPopY,radiusCircleTestYPopN), 
										   (ellipsesForDiagram.get(TEST_Y_SET).getXc()-radiusCircleTestYPopY) + ellipseHorizSemiAxis, 0.0, 0.0));
		EllipseDiagram diag3 = new EllipseDiagram (ellipsesForDiagram3);
		getDisplayPanel().setEulerEllipseDiagram(diag3);
		visStyle[2]= new HeterogeneousChannelStyleOutline("c");
		((HeterogeneousChannelStyleOutline)visStyle[2]).setOutlineColour(Color.pink);
		getDisplayPanel().setSetsLabel(new String[]{"", "", ""});
		getDisplayPanel().setEllipsesVisStyleEuler(TypingUtils.typecastObjArrayToTArray(visStyle, new HeterogeneousChannelStyle[]{}));
	}
	
    protected Ellipse getBestEllipseEnclosing2Regions(Ellipse[] popEllipses, Ellipse[] testEllipses, String ellipseLabel, double reqPopyTestyRegionArea , double reqPopnTestyRegionArea){//, double r_a, double r_b, double area_a, double area_b, double reqAreaAB){
    	// this does not find the best ellipse enclosing 2 horizontally aligned ellipses
    	//    -> instead of trying to find the best fitting ellipse that encloses the 2 input ellipses
    	//       it tries to find the best fitting to best suit the areas of the 2 required regions
    	
    	// assuming also that the 2 inner ellipses are the same except for their centres
    	
    	// assuming popY on left and popN on right
    	
    	
		Ellipse popYEllipse_relabeled = popEllipses[POP_Y_SET].clone();
		popYEllipse_relabeled.setLabel("a");
		Ellipse testYforPopY_relabeled = testEllipses[POP_Y_SET].clone();
		testYforPopY_relabeled.setLabel("b");
		ArrayList<Ellipse> venn2PopyTestyElls = new ArrayList<Ellipse>(Arrays.asList(new Ellipse[]{popYEllipse_relabeled, testYforPopY_relabeled}));
		Ellipse popNEllipse_relabeled = popEllipses[POP_N_SET].clone();
		popNEllipse_relabeled.setLabel("a");
		Ellipse testYforPopN_relabeled = testEllipses[POP_N_SET].clone();
		testYforPopN_relabeled.setLabel("b");
		ArrayList<Ellipse> venn2PopyTestnElls =new ArrayList<Ellipse>(Arrays.asList(new Ellipse[]{popNEllipse_relabeled, testYforPopN_relabeled})); 
		
		//assuming that ellipses are horiz aligned and have same y for centre (and using coordinate system 
		double maxYForIPs = 0;
		Point2D.Double[] ipsVenn2PopyTesty = EllipseDiagramOps.getIPsBet2BaseOverlappingCircles(Math.abs(venn2PopyTestyElls.get(0).getXc()-venn2PopyTestyElls.get(1).getXc()), venn2PopyTestyElls.get(0).getA(), venn2PopyTestyElls.get(1).getA());
		for (Point2D.Double ip : ipsVenn2PopyTesty){
			if (ip.y > maxYForIPs){
				maxYForIPs = ip.y;
			}
		}
		Point2D.Double[] ipsVenn2PopnTesty = EllipseDiagramOps.getIPsBet2BaseOverlappingCircles(Math.abs(venn2PopyTestnElls.get(0).getXc()-venn2PopyTestnElls.get(1).getXc()), venn2PopyTestnElls.get(0).getA(), venn2PopyTestnElls.get(1).getA());
		for (Point2D.Double ip : ipsVenn2PopnTesty){
			if (ip.y > maxYForIPs){
				maxYForIPs = ip.y;
			}
		}
    	
    	
    	// defining the properties of the initial enclosing ellipse
    	double horizDistBetCentresOfInnerEllipses = Math.abs(testEllipses[POP_Y_SET].getXc() - testEllipses[POP_N_SET].getXc());
    	double semiVertAxis = Math.abs(maxYForIPs - testEllipses[POP_Y_SET].getYc()); //assuming that the 2 inner ellipses are same except for centres 
    	double semiHorizAxis = testEllipses[POP_Y_SET].getA() + (horizDistBetCentresOfInnerEllipses/2);
    	Point2D.Double centre = new Point2D.Double (testEllipses[POP_Y_SET].getXc() - testEllipses[POP_Y_SET].getA() + semiHorizAxis, testEllipses[POP_Y_SET].getYc());
    	Ellipse enclosingEllipse = new Ellipse (ellipseLabel, semiHorizAxis, semiVertAxis, centre.x, centre.y, 0.0);
    	
    
        // bisection method
		
		double isZeroL = 1e-7;
    	
		double totalOfReqRegionAreas = reqPopyTestyRegionArea + reqPopnTestyRegionArea; 
		
		double diffReqCurrRegionAreas_lower = 0;
		double diffReqCurrRegionAreas_mid = 0;
		double diffReqCurrRegionAreas_upper = 0;

		double currPopyTestyRegionAreas_lower = 0;
		double currPopyTestyRegionAreas_mid = 0;
		double currPopyTestyRegionAreas_upper = 0;
		
		double currPopnTestyRegionAreas_lower = 0;
		double currPopnTestyRegionAreas_mid = 0;
		double currPopnTestyRegionAreas_upper = 0;
		
		double currTotalRegionAreas_lower = 0;
		double currTotalRegionAreas_mid = 0;
		double currTotalRegionAreas_upper = 0;
    	
		double semiVertAxis_lower = semiVertAxis;
		double semiVertAxis_upper = semiHorizAxis; //assuming that initial semiHorizAxis is always < than semiVertAxis since Test_Y is represented by 2 identical circles with just centre.x which is different
		double semiVertAxis_mid = 0;
		
		double padding = 0;
		semiVertAxis_lower += (padding + isZeroL); 
		semiVertAxis_upper -= (padding + isZeroL); 
		
		

		
		Ellipse enclosingEllipse_trial = enclosingEllipse.clone();
		enclosingEllipse_trial.setLabel("b");
		EllipseDiagram venn2PopyTestyEnclosingEll = new EllipseDiagram(new ArrayList<Ellipse>(Arrays.asList(new Ellipse[]{popEllipses[POP_Y_SET], enclosingEllipse_trial})));
		EllipseDiagram venn2PopnTestyEnclosingEll = new EllipseDiagram(new ArrayList<Ellipse>(Arrays.asList(new Ellipse[]{popNEllipse_relabeled, enclosingEllipse_trial})));
		
		currTotalRegionAreas_lower = venn2PopyTestyEnclosingEll.getZoneAreas().get("ab") + venn2PopnTestyEnclosingEll.getZoneAreas().get("ab");
		

		while(true){ 
					
			semiVertAxis_mid = (semiVertAxis_lower + semiVertAxis_upper)/2;
			
			enclosingEllipse_trial.setB(semiVertAxis_lower);
			venn2PopyTestyEnclosingEll.recomputeZoneAreas();
			venn2PopnTestyEnclosingEll.recomputeZoneAreas();
			currPopyTestyRegionAreas_lower = venn2PopyTestyEnclosingEll.getZoneAreas().get("ab");
			currPopnTestyRegionAreas_lower = venn2PopnTestyEnclosingEll.getZoneAreas().get("ab");
			currTotalRegionAreas_lower = currPopyTestyRegionAreas_lower + currPopnTestyRegionAreas_lower; 
			
			enclosingEllipse_trial.setB(semiVertAxis_mid);
			venn2PopyTestyEnclosingEll.recomputeZoneAreas();
			venn2PopnTestyEnclosingEll.recomputeZoneAreas();
			currPopyTestyRegionAreas_mid = venn2PopyTestyEnclosingEll.getZoneAreas().get("ab");
			currPopnTestyRegionAreas_mid = venn2PopnTestyEnclosingEll.getZoneAreas().get("ab");
			currTotalRegionAreas_mid = currPopyTestyRegionAreas_mid + currPopnTestyRegionAreas_mid; 
			
			enclosingEllipse_trial.setB(semiVertAxis_upper);
			venn2PopyTestyEnclosingEll.recomputeZoneAreas();
			venn2PopnTestyEnclosingEll.recomputeZoneAreas();
			currPopyTestyRegionAreas_upper = venn2PopyTestyEnclosingEll.getZoneAreas().get("ab");
			currPopnTestyRegionAreas_upper = venn2PopnTestyEnclosingEll.getZoneAreas().get("ab");
			currTotalRegionAreas_upper = currPopyTestyRegionAreas_upper + currPopnTestyRegionAreas_upper; 
			
			
			diffReqCurrRegionAreas_lower = totalOfReqRegionAreas - currTotalRegionAreas_lower;
			diffReqCurrRegionAreas_mid = totalOfReqRegionAreas - currTotalRegionAreas_mid;
			diffReqCurrRegionAreas_upper = totalOfReqRegionAreas - currTotalRegionAreas_upper;
			
			if (Math.abs(diffReqCurrRegionAreas_lower) <= isZeroL){
				enclosingEllipse_trial.setB(semiVertAxis_lower);
				venn2PopyTestyEnclosingEll.recomputeZoneAreas();
				venn2PopnTestyEnclosingEll.recomputeZoneAreas();
				
				enclosingEllipse_trial.setB(semiVertAxis_lower);
				enclosingEllipse_trial.setLabel("c");
				return enclosingEllipse_trial;
			} 
			if (Math.abs(diffReqCurrRegionAreas_mid) <= isZeroL){
				enclosingEllipse_trial.setB(semiVertAxis_mid);
				venn2PopyTestyEnclosingEll.recomputeZoneAreas();
				venn2PopnTestyEnclosingEll.recomputeZoneAreas();
				enclosingEllipse_trial.setB(semiVertAxis_mid);
				enclosingEllipse_trial.setLabel("c");
				return enclosingEllipse_trial;
			} 
			if (Math.abs(diffReqCurrRegionAreas_upper) <= isZeroL){
				enclosingEllipse_trial.setB(semiVertAxis_upper);
				venn2PopyTestyEnclosingEll.recomputeZoneAreas();
				venn2PopnTestyEnclosingEll.recomputeZoneAreas();
				enclosingEllipse_trial.setB(semiVertAxis_upper);
				enclosingEllipse_trial.setLabel("c");
				return enclosingEllipse_trial;
			} 
			
			if ((diffReqCurrRegionAreas_lower * diffReqCurrRegionAreas_mid) < 0){ // diff of lower and diff of mid have different signs => when * get -ve ie <0
				semiVertAxis_upper = semiVertAxis_mid;
			} else {       // diff of lower and diff of mid have same signs => when * get +ve ie >0
				semiVertAxis_lower = semiVertAxis_mid;
			} 
			
			if (Math.abs(semiVertAxis_upper-semiVertAxis_lower) <= isZeroL){  //can remove Math.abs because upper must be > lower but leave it to be on the safe siden n// if ((d_upper-d_lower) < isZeroL){
				// Our midpoint is exactly on the root
				enclosingEllipse_trial.setB(semiVertAxis_mid);
				enclosingEllipse_trial.setLabel("c");
				return enclosingEllipse_trial;
			}
		}

	}
    
    
	protected EllipseDiagram computeEulerDiagramForRectangularRegions_ellipses(Rectangle2D.Double[] rectRegions, Rectangle2D.Double[][] maxRectInRegionsForAPEuler, EllipseDiagram areaPropEulerDiagram){
		// the rectangular regions must be passed on as a list in the following order [a, ac, bc, b]
		// note that for ellipses, it is always assumed that the coordinate system used is one such that the origin as at the centre -> this is especially applicable when defining the centres of the ellipses
		// it is assumed that all the ellipses in the diagram have the same centre y that is equal to diagramCentreY
		
		if (rectRegions.length != 4){return null;}   //ideally throw an exception or output an error message but in this case, it will be invoked internally (by an inherited class) so should not be such an issue
		
		ArrayList<Ellipse> ellipsesForSets = new ArrayList<Ellipse>(3);
		int[] setsRefIndex = {POP_Y_SET, POP_N_SET, TEST_Y_SET};
		
		// Get a bounding ellipse for sets a and b respectively
		Rectangle2D.Double bboxForRectRegionsInSet;
		Rectangle2D.Double bboxForMaxRectRegionsInSetForAPEuler;
		int ellipseIndex = 0;
		for (int i=0; i < 4; i+=2){
			ellipsesForSets.add(EulerEllipseVisUtils.getEllipseCircumscribingRectangles2(new Rectangle2D.Double[]{rectRegions[i], rectRegions[i+1]}, ellipsesRefLabel[setsRefIndex[(int)(i/2)]], true));
		}
		
		// Translate ellipses for a and b such that they touch at one point
		double distBetEdgesOfEllipseAandB = (ellipsesForSets.get(setsRefIndex[0]).getXc() + ellipsesForSets.get(setsRefIndex[0]).getA()) - (ellipsesForSets.get(setsRefIndex[1]).getXc() - ellipsesForSets.get(setsRefIndex[1]).getA());
		ellipsesForSets.get(setsRefIndex[0]).setXc(ellipsesForSets.get(setsRefIndex[0]).getXc() - (distBetEdgesOfEllipseAandB/1));
		
		// Translate also the rectangles for regions ac and bc which should reside in ellipse c 
		rectRegions[0].x -= (distBetEdgesOfEllipseAandB/1);
		rectRegions[1].x -= (distBetEdgesOfEllipseAandB/1);
		
		// Find ellipse c 
		ellipsesForSets.add(EulerEllipseVisUtils.getEllipseCircumscribingRectangles2(new Rectangle2D.Double[]{rectRegions[1], rectRegions[2]}, ellipsesRefLabel[setsRefIndex[2]], true));		
		
		return new EllipseDiagram(ellipsesForSets);
	}
	
	protected EllipseDiagram computeEulerDiagramForRectangularRegions_circles(Rectangle2D.Double[] rectRegions, Rectangle2D.Double[][] maxRectInRegionsForAPEuler, EllipseDiagram areaPropEulerDiagram){
		// the rectangular regions must be passed on as a list in the following order [a, ac, bc, b]
		// note that for ellipses, it is always assumed that the coordinate system used is one such that the origin as at the centre -> this is especially applicable when defining the centres of the ellipses
		// it is assumed that all the ellipses in the diagram have the same centre y that is equal to diagramCentreY
		
		if (rectRegions.length != 4){return null;}   //ideally throw an exception or output an error message but in this case, it will be invoked internally (by an inherited class) so should not be such an issue
		
		ArrayList<Ellipse> ellipsesForSets = new ArrayList<Ellipse>(3);
		int[] setsRefIndex = {POP_Y_SET, POP_N_SET, TEST_Y_SET};
		
		// Get a bounding circle (not ellipse) for sets a and b respectively
		for (int i=0; i < 4; i+=2){
			ellipsesForSets.add(EulerEllipseVisUtils.getCircleCircumscribingRectangles(new Rectangle2D.Double[]{rectRegions[i], rectRegions[i+1]}, ellipsesRefLabel[setsRefIndex[(int)(i/2)]], true));
		}
		
		// Translate ellipses for a and b such that they touch at one point
		double distBetEdgesOfEllipseAandB = (ellipsesForSets.get(setsRefIndex[0]).getXc() + ellipsesForSets.get(setsRefIndex[0]).getA()) - (ellipsesForSets.get(setsRefIndex[1]).getXc() - ellipsesForSets.get(setsRefIndex[1]).getA());
		ellipsesForSets.get(setsRefIndex[0]).setXc(ellipsesForSets.get(setsRefIndex[0]).getXc() - (distBetEdgesOfEllipseAandB/1));
		
		// Translate also the rectangles for regions ac and bc which should reside in ellipse c 
		rectRegions[0].x -= (distBetEdgesOfEllipseAandB/1);
		rectRegions[1].x -= (distBetEdgesOfEllipseAandB/1);
		
		// Find ellipse c
		ellipsesForSets.add(EulerEllipseVisUtils.getCircleCircumscribingRectangles(new Rectangle2D.Double[]{rectRegions[1], rectRegions[2]}, ellipsesRefLabel[setsRefIndex[2]], true));		
		
		return new EllipseDiagram(ellipsesForSets);
	}
	
	public static EllipseDiagram computeEulerDiagramForRectangularRegions(Rectangle2D.Double[] rectRegions, Rectangle2D.Double[][] maxRectInRegionsForAPEuler, EllipseDiagram areaPropEulerDiagram){
		// the rectangular regions must be passed on as a list in the following order [a, ac, bc, b]
		// note that for ellipses, it is always assumed that the coordinate system used is one such that the origin as at the centre -> this is especially applicable when defining the centres of the ellipses
		// it is assumed that all the ellipses in the diagram have the same centre y that is equal to diagramCentreY
		
		if (rectRegions.length != 4){return null;}   //ideally throw an exception or output an error message but in this case, it will be invoked internally (by an inherited class) so should not be such an issue
		
		ArrayList<Ellipse> ellipsesForSets = new ArrayList<Ellipse>(3);
		int[] setsRefIndex = {POP_Y_SET, POP_N_SET, TEST_Y_SET};
		
		// Find ellipse c
	    Ellipse ellipseC = EulerEllipseVisUtils.getEllipseCircumscribingRectangles2(new Rectangle2D.Double[]{rectRegions[1], rectRegions[2]}, ellipsesRefLabel[setsRefIndex[2]], true);
	    
		// Translate the rectangles for regions ac and bc which should reside in ellipse c 
		rectRegions[0].x = ellipseC.getXc() - ellipseC.getA() - rectRegions[0].width;
		rectRegions[3].x = ellipseC.getXc() + ellipseC.getA();
		
		// Get a bounding circle (not ellipse) for sets a and b respectively
		for (int i=0; i < 4; i+=2){
			ellipsesForSets.add(EulerEllipseVisUtils.getEllipseCircumscribingRectangles2(new Rectangle2D.Double[]{rectRegions[i], rectRegions[i+1]}, ellipsesRefLabel[setsRefIndex[(int)(i/2)]], true));
		}
		ellipsesForSets.add(ellipseC);
		
		return new EllipseDiagram(ellipsesForSets);
	}
    

	

	public static Rectangle2D.Double[][] getLargestContainedRectsForEulerRegions(EllipseDiagram diagram){
		//*** has its problems esp to find the rectangle for region b
		
		// this excludes the empty zone and thus it returns an array of 4 elements with rectangles for regions [a, ac, bc, b]
		// => this is specific for an euler diagram with 3 curves whereby the first and the last curves (a - index 0 - and b - index 1 - respectively) are circles and touching at one single point, 
		//    such that circle a is on the left of b. (It is because a and b are circles that the x and y of the intersection point of the circle and a 45degs line passing through a point on the diameter is 
		//    (sqrt(2)/2) x distance between intersection point on circle and point on diameter => a and b must be circles for this method to issue the correct rectangles 
		//    Curve c is an ellipse that intersects with both a and b, and since the y-coordinate of the centre of the ellipses is the same for all the ellipses 
		//    (such that their centres differences by the x-coordinate only), then region c is visible and disconnect, but this region is not considered here 
		// => assuming that there is a horizontal line of symmetry running through the centre of the diagram 
		
		ArrayList<Ellipse> ellipses = diagram.getEllipses();
		Ellipse ellipseA = ellipses.get(0);
		Ellipse ellipseB = ellipses.get(1);
		Ellipse ellipseC = ellipses.get(2);
		Rectangle2D.Double[][] rects = new Rectangle2D.Double[4][];
		
		double width = 0 ;
		double height = 0;
		Point2D.Double topRightCoord = new Point2D.Double (0.0, 0.0);
		
		// 1-set regions that is a and b (indexes in array: 0 and 3 respectively)
		
		// ... region a 
		boolean centreAInRegionAC = ( ellipseA.getXc() > (ellipseC.getXc() - ellipseC.getA()) );
		if (centreAInRegionAC){
			double distBetLeftmostEdgesOfEllipses = (ellipseC.getXc()-ellipseC.getA()) - (ellipseA.getXc()-ellipseA.getA()); 
			width = (Math.sqrt(2)/2) * distBetLeftmostEdgesOfEllipses;
			height =  2 * ( (Math.sqrt(2)/2) * distBetLeftmostEdgesOfEllipses );  // or 2 * width;
			topRightCoord.x = (ellipseC.getXc() - ellipseC.getA()) - width;
			topRightCoord.y =  ellipseA.getYc() + (height/2);			
		} else {
			width = ((Math.sqrt(2)/2) * ellipseA.getA()) + (ellipseA.getXc() - (ellipseC.getXc() - ellipseC.getA()));
			height = 2 * ((Math.sqrt(2)/2) * ellipseA.getA());
			topRightCoord.x = ellipseA.getXc() - ((Math.sqrt(2)/2) * ellipseA.getA());
			topRightCoord.y = ellipseA.getYc() + (height/2);          //rem: assuming that the y-coord for all the ellipses is the same -> could have used ellipseC.getYc()- (height/2); 
		}
		rects[0] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
		
		// ... region b 
     	boolean centreBInRegionBC = ( ellipseB.getXc() < (ellipseC.getXc() + ellipseC.getA()) );
		if (centreBInRegionBC){
			double distBetRightmostEdgesOfEllipses = (ellipseB.getXc()+ellipseB.getA()) - (ellipseC.getXc()+ellipseC.getA()); 
			width = (Math.sqrt(2)/2) * distBetRightmostEdgesOfEllipses;
			height =  2 * ( (Math.sqrt(2)/2) * distBetRightmostEdgesOfEllipses );  // or 2 * width;
			topRightCoord.x = ellipseC.getXc() +  ellipseC.getA();
			topRightCoord.y = ellipseC.getYc() + (height/2);			
		} else {
			width = ((Math.sqrt(2)/2) * ellipseB.getA()) + (ellipseB.getXc() - (ellipseC.getXc() + ellipseC.getA()));
			height = 2 * ((Math.sqrt(2)/2) * ellipseB.getA());
			topRightCoord.x = ellipseC.getXc() +  ellipseC.getA();
			topRightCoord.y = ellipseC.getYc() + (height/2);   
		}
		rects[3] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
		
		// 2-set regions that is ac and bc (indexes in array: 1 and 2 respectively)

		// ... region ac 
		double widthOfIntersectRegionAC = Math.abs((ellipseA.getXc()+ellipseA.getA()) - (ellipseC.getXc()-ellipseC.getA()));
		double centreRegionAC_x = (ellipseC.getXc()-ellipseC.getA()) + (widthOfIntersectRegionAC/2);
		double centreRegionAC_y =ellipseA.getYc();
		
		MyLine line45degFromIntersectCentreForEllA = new MyLine(1, -centreRegionAC_x, false);
		ArrayList<Point2D.Double> ipsEllipseAWithLine = ellipseA.getIntPnts_WithLine(line45degFromIntersectCentreForEllA);
		Point2D.Double ipEllipseAWithLine = (ipsEllipseAWithLine.get(0).getX() > ipsEllipseAWithLine.get(1).getX()) ? ipsEllipseAWithLine.get(0) : ipsEllipseAWithLine.get(1);
		double distIPEllipseAWithLineToCentre_x = Math.abs(ipEllipseAWithLine.x - centreRegionAC_x);
		double distIPEllipseAWithLineToCentre_y = Math.abs(ipEllipseAWithLine.y - centreRegionAC_y);
		
		MyLine line45degFromIntersectCentreForEllC = new MyLine(-1, centreRegionAC_x, false);
		ArrayList<Point2D.Double> ipsEllipseCWithLine = ellipseC.getIntPnts_WithLine(line45degFromIntersectCentreForEllC);
		Point2D.Double ipEllipseCWithLine = (ipsEllipseCWithLine.get(0).getX() < ipsEllipseCWithLine.get(1).getX()) ? ipsEllipseCWithLine.get(0) : ipsEllipseCWithLine.get(1);
		double distIPEllipseCWithLineToCentre_x = Math.abs(ipEllipseCWithLine.x - centreRegionAC_x);
		double distIPEllipseCWithLineToCentre_y = Math.abs(ipEllipseCWithLine.y - centreRegionAC_y);
		
		if (distIPEllipseCWithLineToCentre_y < distIPEllipseAWithLineToCentre_y){
			double[] ellipseA_xs = ellipseA.getX(centreRegionAC_y+distIPEllipseCWithLineToCentre_y);
			width = distIPEllipseCWithLineToCentre_x + Math.abs(((ellipseA_xs[0] > ellipseA_xs[1]) ? ellipseA_xs[0] : ellipseA_xs[1]) - centreRegionAC_x);
			height = 2*distIPEllipseCWithLineToCentre_y;
			topRightCoord.x = centreRegionAC_x - distIPEllipseCWithLineToCentre_x;
		} else {
			double[] ellipseC_xs = ellipseC.getX(centreRegionAC_y+distIPEllipseAWithLineToCentre_y);
			double distCentreACtoEllipseC = Math.abs(((ellipseC_xs[0] < ellipseC_xs[1]) ? ellipseC_xs[0] : ellipseC_xs[1]) - centreRegionAC_x);
			width = distIPEllipseAWithLineToCentre_x + distCentreACtoEllipseC;
			height = 2*distIPEllipseAWithLineToCentre_y;
			topRightCoord.x = centreRegionAC_x - distCentreACtoEllipseC;
		}
		topRightCoord.y = centreRegionAC_y + (height/2); 
		rects[1] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
		
		
		// ... region bc 
		double widthOfIntersectRegionBC = Math.abs( (ellipseC.getXc()+ellipseC.getA()) - (ellipseB.getXc()-ellipseB.getA()) );
		double centreRegionBC_x = (ellipseB.getXc()-ellipseB.getA()) + (widthOfIntersectRegionBC/2);
		double centreRegionBC_y =ellipseB.getYc();
		
		MyLine line45degFromIntersectCentreForEllB = new MyLine(-1, centreRegionBC_x, false);
		ArrayList<Point2D.Double> ipsEllipseBWithLine = ellipseB.getIntPnts_WithLine(line45degFromIntersectCentreForEllB);
		Point2D.Double ipEllipseBWithLine = (ipsEllipseBWithLine.get(0).getX() < ipsEllipseBWithLine.get(1).getX()) ? ipsEllipseBWithLine.get(0) : ipsEllipseBWithLine.get(1);
		double distIPEllipseBWithLineToCentre_x = Math.abs(ipEllipseBWithLine.x - centreRegionBC_x);
		double distIPEllipseBWithLineToCentre_y = Math.abs(ipEllipseBWithLine.y - centreRegionBC_y);
		
		line45degFromIntersectCentreForEllC = new MyLine(1, -centreRegionBC_x, false);
		ipsEllipseCWithLine = ellipseC.getIntPnts_WithLine(line45degFromIntersectCentreForEllC);
		ipEllipseCWithLine = (ipsEllipseCWithLine.get(0).getX() > ipsEllipseCWithLine.get(1).getX()) ? ipsEllipseCWithLine.get(0) : ipsEllipseCWithLine.get(1);
		distIPEllipseCWithLineToCentre_x = Math.abs(ipEllipseCWithLine.x - centreRegionBC_x);
		distIPEllipseCWithLineToCentre_y = Math.abs(ipEllipseCWithLine.y - centreRegionBC_y);
		
		if (distIPEllipseBWithLineToCentre_y < distIPEllipseCWithLineToCentre_y){
			double[] ellipseC_xs = ellipseC.getX(centreRegionBC_y+distIPEllipseBWithLineToCentre_y);
			width = distIPEllipseBWithLineToCentre_x + Math.abs(((ellipseC_xs[0] > ellipseC_xs[1]) ? ellipseC_xs[0] : ellipseC_xs[1]) - centreRegionBC_x);
			height = 2*distIPEllipseBWithLineToCentre_y;
			topRightCoord.x = centreRegionBC_x - distIPEllipseBWithLineToCentre_x;
		} else {
			double[] ellipseB_xs = ellipseB.getX(centreRegionBC_y+distIPEllipseCWithLineToCentre_y);
			double distCentreBCtoEllipseBx = Math.abs(((ellipseB_xs[0] < ellipseB_xs[1]) ? ellipseB_xs[0] : ellipseB_xs[1]) - centreRegionBC_x);
			width = distIPEllipseCWithLineToCentre_x + distCentreBCtoEllipseBx;
			height = 2*distIPEllipseCWithLineToCentre_y;
			topRightCoord.x = centreRegionBC_x - distCentreBCtoEllipseBx;
		}
		topRightCoord.y = centreRegionBC_y + (height/2); 
		
		
		//this ensures the left side is within the region
		MyLine vertleftline = new MyLine(Double.POSITIVE_INFINITY, topRightCoord.x, false);
		ArrayList<Point2D.Double> ipsEllipseBWithVertLeftLine = ellipseB.getIntPnts_WithLine(vertleftline);
		topRightCoord.y = Math.max(ipsEllipseBWithVertLeftLine.get(0).y,ipsEllipseBWithVertLeftLine.get(1).y);
		height = Math.abs(ipsEllipseBWithVertLeftLine.get(0).y - ipsEllipseBWithVertLeftLine.get(1).y);
		
		rects[2] = new Rectangle2D.Double[]{new Rectangle2D.Double(topRightCoord.x, topRightCoord.y, width, height)};
		
		return rects;
	}
	
	

	
	protected double[] composeRegionSizes(double[] bayesProbabs, ShowRegionSizeMode mode, int totalPopFreq){
	
		// the non-empty regions for this euler diagram (=> its abstract description) include:   a, b, ac, bc
		// when a = Pop_y, b = Pop_n, c = Test_y
		// => return the region sizes accordingly and must be in the following order a,b,c,ab,ac,bc,abc
		
		double regionBC = bayesProbabs[TEST_Y_GIVEN_POP_N];
		double regionAC = bayesProbabs[TEST_Y_GIVEN_POP_Y];
		double regionB = bayesProbabs[TEST_N_GIVEN_POP_N];
		double regionA = bayesProbabs[TEST_N_GIVEN_POP_Y];
		
		double[] regionSizesAsProbabs = new double[]{regionA, regionB, regionAC, regionBC};
		
		return mode.translateProbabsAsMode(regionSizesAsProbabs, totalPopFreq);
	}
	
	protected String[] composeRegionSizeLabels (double[] bayesProbabs, ShowRegionSizeMode mode, int totalPopFreq){
		final int DP = 2;
		final int MAX_SF_PRIOR_DP = 15;
			
		double[] regionSizes = composeRegionSizes (bayesProbabs, mode, totalPopFreq);
		double[] regionSizesRnded = new double[regionSizes.length];
		
		int i = 0;
		for (double rs : regionSizes){
			regionSizesRnded[i] = MathUtils.roundToDps(rs, MAX_SF_PRIOR_DP, DP);
			i++;
		}
		return translateValuesToStringForMode(regionSizesRnded, mode);
	}
	
	
	
}
