/* 
 * 
 * eulerGlyphs v0.9.0
 * 
 * 2013-12-18
 *
 * 
 * 
 * eulerGlyphs -- Drawing Area-Proportional Euler Diagrams With Glyphs	    
 * 		http://www.eulerdiagrams.org/eulerGlyphs
 * 
 * 
 * 		Copyright (C) 2012-2013, Luana Micallef. 
 * 		All rights reserved.
 * 		
 * 
 * 		This file is part of eulerGlyphs.
 * 			
 * 		eulerGlyphs is free software: you can redistribute it and/or modify
 * 		it under the terms of the GNU General Public License as published 
 * 		by the Free Software Foundation, either version 3 of the License, 
 * 		or (at your option) any later version.
	
 * 		eulerGlyphs is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 		GNU General Public License for more details.
	
 * 		A copy of the GNU General Public License is provided with 
 * 		eulerGlyphs (in a file named 'COPYING'). Alternatively, see 
 * 		<http://www.gnu.org/licenses/gpl.html>.
 * 			
 */

 


package visforfallacy;


import uicomponent.vispanel.VisPanel;
import utils.MathUtils;


/**
 * An abstract class for all the proposed visualizations to avoid different types of logical fallacies
 * 
 */


public abstract class VisForFallacy {
	
	public static enum ShowRegionSizeMode {
		DONOT_SHOW { double[] translateProbabsAsMode(double[] probabs, int totalPopFreq) { return null; } 
        			 String translateValueToStringForMode(double value) { return null; } },
		
		SHOW_AS_FREQ { double[] translateProbabsAsMode(double[] probabs, int totalPopFreq) { return translateProbabsToFreqs(probabs, totalPopFreq); } 
		               String translateValueToStringForMode(double value) { return Double.toString(value); } }, 
		               
		SHOW_AS_PROBAB_PERCENT { double[] translateProbabsAsMode(double[] probabs, int totalPopFreq) { return translateProbabsToPercents(probabs); } 
        						 String translateValueToStringForMode(double value) { return (Double.toString(value)+"%"); } }, 
	    
        SHOW_AS_PROBAB_DECIMAL { double[] translateProbabsAsMode(double[] probabs, int totalPopFreq) { return probabs; } 
        						 String translateValueToStringForMode(double value) { return Double.toString(value); } };
        						 
        abstract double[] translateProbabsAsMode(double[] probabs, int totalPopFreq);
        abstract String translateValueToStringForMode(double value);
	};

	
	// Fields 
	/**
	 * The known probabilities for the problem that should be visualized
	 */
	protected double[] inputProbabs;
	/**
	 * Labels for the visualization
	 */
	protected String[] labels;
	/**
	 * The VisPanel instance where the visualization should be displayed
	 */
	protected VisPanel displayPanel;
	
	
	// Getters
    public double[] getInputProbabs(){
		return this.inputProbabs;
	}
    public String[] getLabels(){
		return this.labels;
	}
	public VisPanel getDisplayPanel(){
		return this.displayPanel;
	}
	
	// Setters
	public void setInputProbabs(double[] inputProbabs) throws Exception{
		if ((inputProbabs == null) || (inputProbabs.length == 0)){
			throw new Exception ("The probabilities that should be known for the problem are not available");
		} else {
			this.inputProbabs = inputProbabs;
		}
	}
	public void setLabels(String[] labels) throws Exception{
		this.labels = labels;
	}
	public void setDisplayPanel (VisPanel displayPanel){
		this.displayPanel = displayPanel;
	}

	
	
	// Static Methods
    // ... utilities 
	/**
	 * Translates a probability (in decimal form as a value between 0 and 1) to a natural frequency based on a given population size
	 * 
	 * @param probab the probability (in decimal form as a value between 0 and 1) to be converted to frequency
	 * @param popsize the size of the population
	 * @return the translated frequency for the given probability and population size 
	 */
	protected static double translateProbabToFreq (double probab, double popsize){
		return (probab * popsize);
	}
	/**
	 * Translates a probability (in decimal form as a value between 0 and 1) to a natural frequency based on a calculated population size
	 * 
	 * @param probab the probability (in decimal form as a value between 0 and 1) to be converted to frequency
	 * @return the translated frequency for the given probability and a calculated minimum population size to get whole numbers for frequencies  
	 */
	protected static double translateProbabToFreq (double probab){
		return (probab * MathUtils.getMutliplyFactorToConvertDecimalToWholeNumber(probab));
	}
	/**
	 * Translates a probability (in decimal form as a value between 0 and 1) to a percentage (out of 100)
	 * 
	 * @param probab the probability (in decimal form as a value between 0 and 1) to be converted to a percentage (out of 100)
	 * @return the percentage value (out of 100) for the given probability
	 */
	protected static double translateProbabToPercent (double probab){
		return (probab * 100);
	}
	/**
	 * Translates an array of probabilities (in decimal form as a value between 0 and 1) to an array of natural frequencies based on a given population size
	 * 
	 * @param probabs an array containing the probabilities (in decimal form as a value between 0 and 1) to be converted to frequencies
	 * @param popsize the size of the population
	 * @return an array containing the translated frequencies for the given probabilities and population size 
	 */
	protected static double[] translateProbabsToFreqs (double[] probabs, double popsize){
		double[] freqs = new double[probabs.length];
		for (int i=0; i<probabs.length; i++){
			freqs[i] = translateProbabToFreq (probabs[i], popsize);
		}
		return freqs;
	}
	/**
	 * Translates an array of probabilities (in decimal form as a value between 0 and 1) to an array of natural frequencies based on a calculated population size
	 * which ensures that all the frequencies are whole numbers
	 * 
	 * @param probabs an array containing the probabilities (in decimal form as a value between 0 and 1) to be converted to frequencies
	 * @return an array containing the translated frequencies for the given probabilities and a calculated minimum population size to get whole numbers for frequencies 
	 */
	protected static double[] translateProbabsToFreqs (double[] probabs, double[] probabsToFindMultFactor){
		double[] freqs = new double[probabs.length];
		
		// calculate a minimum population size to get whole numbers for frequencies
		double smallestProbab = MathUtils.getMinInArray(probabsToFindMultFactor);
		int probabMultFactor = MathUtils.getMutliplyFactorToConvertDecimalToWholeNumber(smallestProbab);
		
		for (int i=0; i<probabs.length; i++){
			freqs[i] = probabs[i] * probabMultFactor;
		}
		return freqs;
	}
	/**
	 * Translates an array of probabilities (in decimal form as a value between 0 and 1) to an array of percentages (out of 100)
	 * 
	 * @param probabs an array containing the probabilities (in decimal form as a value between 0 and 1) to be converted to percentages (out of 100)
	 * @return an array containing the percentage values (out of 100) for the given probabilities
	 */
	protected static double[] translateProbabsToPercents (double[] probabs){
		double[] percents = new double[probabs.length];
		for (int i=0; i<probabs.length; i++){
			percents[i] = translateProbabToPercent(probabs[i]);
		}
		return percents;
	}
	
	
	/**
	 * Translates values to string according to the required representation mode
	 * 
	 * @param values an array of the values to be converted to String according to the required representation mode
	 * @return an array containing the translated String values for the given double values and representation mode
	 */
	protected static String[] translateValuesToStringForMode(double[] values, ShowRegionSizeMode mode){
		String[] stringValues = new String[values.length];
		int i = 0;
		for(double v: values){
			stringValues[i] = mode.translateValueToStringForMode(v);
			i++;
		}
		return stringValues;
	}
	
	
	
	
	/**
	 * Generates the visualization for the fallacy problem 
	 * 
	 * @throws Exception An exception is throw when the probabilities that should be known for the problem are not available
	 */
	public void generate(boolean showSetLabels, boolean showLegend) throws Exception{
		generate (showSetLabels, ShowRegionSizeMode.DONOT_SHOW, showLegend, 0);
	}
	public void generate(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq) throws Exception{
		if ((inputProbabs == null) || (inputProbabs.length == 0)){
			throw new Exception ("The probabilities that should be known for the problem are not available");
		} else {
			generateSafely(showSetLabels, showRegionSizeMode, showLegend, totalPopFreq);
		}
	}
	
	
	// Abstract Methods
	/**
	 * Generates the visualization for the fallacy problem 
	 * without checking whether all the probabilities that should be known for the problem are available or not 
	 * 
	 * {@link #generate()} carries out this check. {@link #generate()} is actually the one which is publicly accessible.
	 */
	protected abstract void generateSafely(boolean showSetLabels, ShowRegionSizeMode showRegionSizeMode, boolean showLegend, int totalPopFreq);
}
